/****************************************************************/
/*                                                              */
/*      fkt.c                                                   */
/*                                                              */
/*      This file is a part of kpl - a program for graphical    */
/*      presentation of data sets and functions.                */
/*                                                              */
/*      Copyright (C) 1999 Werner Stille                        */
/*      <stille@uni-freiburg.de>                                */
/*                                                              */
/*      Released under the GPL; see file LICENSE for details.   */
/*                                                              */
/****************************************************************/

/* Example functions for kpl */

/* Compiles also on other platforms.;-) */

#if defined(_Windows) || defined(__WIN32__)
  #include <windows.h>
#endif

#ifdef __GNUC__
  #define _api
  #ifndef __WIN32__
    #define FAR
  #endif
#else
  #define _api FAR WINAPI _export
#endif

#include <math.h>

/****************************************************************/
/*                                                              */
/*    polynom(x, p)                                             */
/*      calculates polynomial                                   */
/*                                                              */
/*    Returns: p[0] + p[1]*x * p[2]*x*x + p[3]*x*x*x + ...      */
/*                                                              */
/****************************************************************/

double _api polynom(double x, double FAR *p)
{
  int i;
  double f = 0.0;

  if (x)
    for (i = 19; i >= 0; i--) {
      if (f)
        f *= x;
      if (p[i])
        f += p[i];
    }
  else
    f = p[0];
  return f;
}

/****************************************************************/
/*                                                              */
/*    foursynth(x, p)                                           */
/*      calculates Fourier synthesis                            */
/*                                                              */
/*    Returns: p[0] + p[2]*cos(x-p[1]) + p[3]*sin(x-p[1])       */
/*               + p[4]*cos(2*(x-p[1])) + p[5]*sin(2*(x-p[1]))  */
/*               + p[6]*cos(3*(x-p[1])) + p[7]*sin(3*(x-p[1]))  */
/*               + p[8]*cos(2*(x-p[1])) + ...                   */
/*                                                              */
/****************************************************************/

double _api foursynth(double x, double FAR *p)
{
  int i, j;
  double f = 0, a;

  x -= p[1];
  for (i = 2; i < 19; i += 2) {
    j = i + 1;
    a = (j >> 1) * x;
    if (p[i])
      f += p[i] * cos(a);
    if (p[j])
      f += p[j] * sin(a);
  }
  f += p[0];
  return f;
}

/****************************************************************/
/*                                                              */
/*    gaussian(x, p)                                            */
/*      calculates sum of Gaussians                             */
/*                                                              */
/*    Returns: p[0] + p[1]*exp(((x-p[2])/p[3])^2)               */
/*               + p[4]*exp(((x-p[5])/p[6])^2)                  */
/*               + p[7]*exp(((x-p[8])/p[9])^2) + ...            */
/*                                                              */
/****************************************************************/

double _api gaussian(double x, double FAR *p)
{
  int i;
  double f = 0, t;

  for (i = 1; i < 18; i += 3) {
    if (p[i] && p[i + 2]) {
      t = (x - p[i + 1]) / p[i + 2];
      if (fabs(t) < 37.0)
        f += p[i] * exp(-t * t);
    }
  }
  f += p[0];
  return f;
}

/****************************************************************/
/*                                                              */
/*    exponential(x, p)                                         */
/*      calculates sum of exponentials                          */
/*                                                              */
/*    Returns: p[0] + p[1]*exp(p[2]*x) + p[3]*exp(p[4]*x)       */
/*                + p[5]*exp(p[6]*x) + ...                      */
/*                                                              */
/****************************************************************/

double _api exponential(double x, double FAR *p)
{
  int i;
  double f = 0;

  for (i = 1; i < 18; i += 2)
    if (p[i])
      f += p[i] * exp(p[i + 1] * x);
  f += p[0];
  return f;
}

/****************************************************************/
/*                                                              */
/*    powers(x, p)                                              */
/*      calculates sum of powers of x                           */
/*                                                              */
/*    Returns: p[0]*x^p[1] + p[2]*x^p[3] + p[4]*x^p[5] + ...    */
/*                                                              */
/****************************************************************/

double _api powers(double x, double FAR *p)
{
  int i;
  double f = 0;

  for (i = 0; i < 19; i += 2)
    if (p[i])
      f += p[i] * pow(x, p[i + 1]);
  return f;
}
