/***************************************************************************
                          funcdlg.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 1999 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FUNCDLG_H
#define FUNCDLG_H

#include <stdio.h>
#include <qdialog.h>
#include "kplots.h"

class KplDoc;
class KColorButton;
class QLineEdit;
class QListBox;
class QSpinBox;

/**
  * Choose function dialog class. Provides a dialog for the selection
  * of a function from list of functions in a shared library.
  * @author Werner Stille
  */
class ChooseFuncDlg : public QDialog {
  Q_OBJECT

public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param f pointer to stream opened for input from a file containing
   * a list of functions obtained using nm.
   * @param func pointer to QLineEdit widget for the function name.
   */
  ChooseFuncDlg(QWidget *parent, FILE *f, QLineEdit *func);

protected slots:
  /**
   * Accepts selection and leaves the dialog.
   * @param i index of selected function
   */
  void slotList(int i);
  /** Accepts selection and leaves the dialog. */
  void slotOK();

private:
  QLineEdit *func;
  QListBox *funcList;
};

/**
  * Edit parameter dialog class. Provides a dialog for the editing
  * the parameters of a function.
  * @author Werner Stille
  */
class EditParameterDlg : public QDialog {
  Q_OBJECT

public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param p parameter array.
   * @param file absolute path of the function library.
   * @param func function name.
   */
  EditParameterDlg(QWidget *parent, double *p, const char *file,
                   const char *func);

protected slots:
  /** Accepts changes and leaves the dialog. */
  void slotOK();

private:
  double *p;
  QLineEdit *par[20];
};

/**
  * Function settings dialog class. Provides a dialog for the settings
  * of an ordinary function.
  * @author Werner Stille
  */
class FuncDlg : public QDialog {
  Q_OBJECT

public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   * @param fd pointer to function data structure.
   */
  FuncDlg(QWidget *parent, KplDoc *model, PFUNDATA fd);
  /** Destructor. */
  ~FuncDlg();

protected:
  /**
   * Copies settings to temporary function data structure.
   * @param ft pointer to temporary data structure.
   */
  bool getTemp(PFUNDATA ft);
  /**
   * Copies settings to function data structure.
   * @param accept true for leaving the dialog, false for applying.
   * @return false for error.
   */
  void getValues(bool accept);

protected slots:
  /** Displays the dialog for selection of a library file to load. */
  void slotFile();
  /** Displays the dialog for selection of a function. */
  void slotFunc();
  /** Displays the dialog for selection of a parameter file to read. */
  void slotGetParameter();
  /** Displays the dialog for parameter editing. */
  void slotEditParameter();
  /** Displays the dialog for selection of a parameter file to save. */
  void slotSaveParameter();
  /** Saves function value table into a data file. */
  void slotExportValues();
  /** Displays the dialog for selection of lines and markers. */
  void slotSymbol();
  /** Applies settings and leaves the dialog. */
  void slotOK();
  /** Applies settings without leaving the dialog. */
  void slotApply();
  /** Displays help for the function settings dialog. */
  void slotHelp();

signals:
  /**
    * Emitted when applied changes require updating the menus and repainting the
    * page view without leaving the dialog.
    */
  void modelChanged();

private:
  KplDoc *m;
  PFUNDATA fd, fdt;
  QLineEdit *fileName, *func, *exMin, *exMax, *exStep, *efx, *efy;
  QSpinBox *symb;
  KColorButton *colData;
};

/**
  * Parametric function settings dialog class. Provides a dialog for the settings
  * of parametric functions.
  * @author Werner Stille
  */
class ParFuncDlg : public QDialog {
  Q_OBJECT

public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param model pointer to KplDoc model object.
   * @param fd pointer to parametric function data structure.
   */
  ParFuncDlg(QWidget *parent, KplDoc *model, PPARFUNDATA fd);
  /** Destructor. */
  ~ParFuncDlg();

protected:
  /**
   * Copies settings to temporary parametric function data structure.
   * @param ft pointer to temporary data structure.
   * @return false for error.
   */
  bool getTemp(PPARFUNDATA ft);
  /**
   * Copies settings to parametric function data structure.
   * @param accept true for leaving the dialog, false for applying.
   */
  void getValues(bool accept);

protected slots:
  /** Displays the dialog for selection of a library file to load for the x axis. */
  void slotFilex();
  /** Displays the dialog for selection of a library file to load for the y axis. */
  void slotFiley();
  /** Displays the dialog for selection of a function for the x axis. */
  void slotFuncx();
  /** Displays the dialog for selection of a function for the y axis. */
  void slotFuncy();
  /** Displays the dialog for selection of a parameter file to read for the x axis function. */
  void slotGetParameterx();
  /** Displays the dialog for selection of a parameter file to read for the y axis function. */
  void slotGetParametery();
  /** Displays the dialog for parameter editing for the x axis function. */
  void slotEditParameterx();
  /** Displays the dialog for parameter editing for the y axis function. */
  void slotEditParametery();
  /** Displays the dialog for selection of a parameter file to save for the x axis function. */
  void slotSaveParameterx();
  /** Displays the dialog for selection of a parameter file to save for the y axis function. */
  void slotSaveParametery();
  /** Saves function value table into a data file. */
  void slotExportValues();
  /** Displays the dialog for selection of lines and markers. */
  void slotSymbol();
  /** Applies settings and leaves the dialog. */
  void slotOK();
  /** Applies settings without leaving the dialog. */
  void slotApply();
  /** Displays help for the parametric function settings dialog. */
  void slotHelp();

signals:
  /**
    * Emitted when applied changes require updating the menus and repainting the
    * page view without leaving the dialog.
    */
  void modelChanged();

private:
  KplDoc *m;
  PPARFUNDATA fd, fdt;
  QLineEdit *fileNamex, *funcx, *fileNamey, *funcy, *etMin, *etMax,
            *etStep, *efx, *efy;
  QSpinBox *symb;
  KColorButton *colData;
};

#endif
