/***************************************************************************
                          kgraph.cpp  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 1999 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <qpaintdevicemetrics.h>
#include <qpixmap.h>
#include <math.h>
#include "kgraph.h"

#ifndef min
  #define min(x,y) ((x) < (y) ? (x) : (y))
#endif
#ifndef max
  #define max(x,y) ((x) > (y) ? (x) : (y))
#endif

static const char face[3][13] = {"Helvetica", "Symbol", "ZapfDingbats"};

KGraph::KGraph() : QPainter()
{
}

void KGraph::r2i(double x, double y, int *ix, int *iy)
{
  xcur = x;
  ycur = y;
  *ix = minx + int(xscal * (x - xmin));
  *iy = miny + int(yscal * (y - ymin));
}

int KGraph::i2cm(int ix, int iy, double *xcm, double *ycm)
{
  *xcm = (ix - minx0) / (scalx * factor);
  *ycm = (iy - miny0) / (scaly * factor);
  return ((ix < minx) || (ix > maxx) || (iy > miny) || (iy < maxy));
}

void KGraph::i2r(int ix, int iy, double fxn, double fyn,
                 double *xr, double *yr)
{
  *xr = ((ix - minx) / xscal + xmin) / fxn;
  *yr = ((iy - miny) / yscal + ymin) / fyn;
}

void KGraph::cm2r(double xcm, double ycm, double fxn, double fyn,
                  double *xr, double *yr)
{
  i2r(int(xcm * scalx * factor) + minx0,
      int(ycm * scaly * factor) + miny0, fxn, fyn, xr, yr);
}

void KGraph::move(double x, double y)
{
  r2i(x, y, &kxcur, &kycur);
  moveTo(kxcur, kycur);
}

void KGraph::draw(double x, double y)
{
  int i, j;
  double x1, y1, d0, d, f;

  if ((x != xcur) || (y != ycur)) {
    if (dash[1] == dash[0]) {
      r2i(x, y, &kxcur, &kycur);
      lineTo(kxcur, kycur);
    } else {
      dcur = dcur - dash[3] * (int) (dcur / dash[3]);
      x1 = (x - xcur) * xscal / scalx;
      y1 = (y - ycur) * yscal / scaly;
      d0 = sqrt(x1 * x1 + y1 * y1);
      d = d0;
      x1 = xcur;
      y1 = ycur;
      i = 0;
      for (j = 0; j < 3; j++)
        if (dcur >= dash[j])
          i = j + 1;
      while ((dcur + d) > dash[i]) {
        f = (dash[i] - dcur) / d;
        if (f) {
          d = d * (1.0 - f);
          xcur = x - d * (x - x1) / d0;
          ycur = y - d * (y - y1) / d0;
          dcur = dash[i];
          if (i % 2)
            move(xcur,ycur);
          else {
            r2i(xcur, ycur, &kxcur, &kycur);
            lineTo(kxcur, kycur);
          }
        }
        if (i == 3)
          dcur = 0.0;
        i = ++i % 4;
      }
      xcur = x;
      ycur = y;
      if (i % 2)
        move(xcur, ycur);
      else {
        r2i(xcur, ycur, &kxcur, &kycur);
        lineTo(kxcur, kycur);
      }
      dcur += d;
    }
  }
}

void KGraph::plInit(const QPaintDevice *pd, int offLeft, int offTop)
{
  QPaintDeviceMetrics pdm(pd);

  begin(pd);
  eraseRect(window());
  scalx = 10.0 * pdm.width() / pdm.widthMM();
  scaly = -10.0 * pdm.height() / pdm.heightMM();
  xscal = scalx;
  yscal = scaly;
  minx0 = minx = -offLeft;
  maxx = pdm.width() + minx;
  miny0 = miny = pdm.height() + offTop;
  maxy = offTop;
  dlen = (maxx - minx) / xscal;
  dhite = (maxy - miny) / yscal;
  factor = 1.0;
  xmin = ymin = 0.0;
  xcur = xmin;
  ycur = ymin;
  xmax = dlen;
  ymax = dhite;
  relang = 0.0;
  relsin = 0.0;
  relcos = 1.0;
  relsiz = 1.0;
  textdir = 0.0;
  linetype(1);
  pen1.setWidth(maxx / 240);
  pen2.setWidth(maxx / 480);
  pen3.setWidth(maxx / 339);
  setPen(pen3);
  QFont fnt = font();
  fnt.setFamily(face[0]);
  fnt.setPointSize(max(int(0.0355 * maxx), 1));
  setFont(fnt);
  move(xcur, ycur);
}

void KGraph::format(double x, double y)
{
  xmin = ymin = 0.0;
  xmax = x;
  ymax = y;
  minx = minx0;
  miny = miny0;
  xscal = scalx * factor;
  yscal = scaly * factor;
  maxx = minx + int(xscal * xmax);
  maxy = miny + int(yscal * ymax);
  xcur = xmin;
  ycur = ymin;
  setClipRect(0, maxy, maxx - minx, miny - maxy);
  setClipping(0);
}

void KGraph::setColFrame(const QColor & icol1)
{
  pen1.setColor(icol1);
}

void KGraph::setColGrid(const QColor & icol2)
{
  pen2.setColor(icol2);
}
 
void KGraph::setColData(const QColor & icol3)
{
  pen3.setColor(icol3);
  setPen(pen3);
} 
 
void KGraph::setCol(const QColor & icol1, const QColor & icol2,
                    const QColor & icol3)
{
  setColFrame(icol1);
  setColGrid(icol2);
  setColData(icol3);
}

double KGraph::relsize(double r)
{
  double old;

  old = relsiz;
  relsiz = r;
  pen1.setWidth(int(0.00416667 * relsiz * (maxx - minx)));
  pen2.setWidth(int(0.00208333 * relsiz * (maxx - minx)));
  pen3.setWidth(int(0.00295 * relsiz * (maxx - minx)));
  setPen(pen3);
  QFont fnt = font();
  fnt.setPointSize(max(int(0.0355 * relsiz * (maxx - minx)), 1));
  setFont(fnt);
  return old;
}

void KGraph::Window(double x0, double xl, double y0, double yl)
{
  int dx;

  xmin = x0;
  xmax = x0 + xl;
  ymin = y0;
  ymax = y0 + yl;
  minx = minx0 + int(xscal * xmin);
  maxx = minx + int(xscal * xl);
  miny = miny0 + int(yscal * ymin);
  maxy = miny + int(yscal * yl);
  xcur = xmin;
  ycur = ymin;
  move(xcur, ycur);
  dcur = 0.0;
  dx = int(0.00416667 * relsiz * (maxx - minx));
  pen1.setWidth(dx);
  pen2.setWidth(int(0.00208333 * relsiz * (maxx - minx)));
  pen3.setWidth(int(0.00295 * relsiz * (maxx - minx)));
  setPen(pen3);
  QFont fnt = font();
  fnt.setPointSize(max(int(0.0355 * relsiz * (maxx - minx)), 1));
  setFont(fnt);
  setClipRect(minx + (dx >> 1) , maxy + (dx >> 1), maxx - minx - dx,
              miny - maxy - dx);
  setClipping(0);
} 

void KGraph::scale(double xl, double xr, double yu, double yo)
{
  xmin = xl;
  xmax = xr;
  ymin = yu;
  ymax = yo;
  xscal = (maxx - minx) / (xmax - xmin);
  yscal = (maxy - miny) / (ymax - ymin);
  xcur = xmin;
  ycur = ymin;
  move(xcur, ycur);
}

void KGraph::setdig(int nx, int ny)
{
  ndigx = nx;
  ndigy = ny;
}

void KGraph::frame(void)
{
  setPen(pen1);
  drawRect(minx, maxy, maxx - minx, miny - maxy);
  setPen(pen3);
} 

void KGraph::raster(double xtic, double ytic, int intx, int inty, int mode)
{
  int i, l, ix, iy;
  double dx = 0.0, dy, x, y, xm, ym;
  char label[11];

  if (mode) {
    setPen(pen2);
    dx = 0.99999 * (xmax - xmin);
    dy = 0.99999 * (ymax - ymin);
    if (mode <= 2) {
      dx *= 0.02;
      dy = -dx * xscal / yscal;
    }
    for (x = xmin; x < xmax;) {
      if (mode <= 2  && intx > 1) {
        xm = x;
        for (i = 1; i < intx; i++)
          if ((xm += xtic / intx) < xmax) {
            move(xm, ymin);
            draw(xm, ymin + 0.5 * dy);
          }
      }
      if ((x += xtic) < xmax) {
        move(x, ymin);
        draw(x, ymin + dy);
      }
    }
    for (y = ymin; y < ymax;) {
      if (mode <= 2  && intx > 1) {
        ym = y;
        for (i = 1; i < inty; i++)
          if ((ym += ytic / inty) < ymax) {
            move(xmin, ym);
            draw(xmin + 0.5 *dx, ym);
          }
      }
      if ((y += ytic) < ymax) {
        move(xmin, y);
        draw(xmin + dx, y);
      }
    }
  }
  setPen(pen1);
  if ((mode == 2) || (mode == 4)) {
    y = ymin + 0.03225 * relsiz * (xmax - xmin) * xscal / yscal;
    for (x = xmin; x <= (xmax + 1.0e-3 * dx);) {
      if (x > xmax)
        x = xmax;
      l = prenum(x, ndigx, label);
      r2i(x, y, &ix, &iy);
      drawText(ix, iy, 0, 0, AlignCenter | DontClip, label, l);
      x += xtic;
    }
    x = xmin - 0.025 *relsiz * (xmax - xmin);
    dy = -0.0125 * relsiz * (xmax - xmin) * xscal / yscal;
    for (y = ymin; y <= (ymax + 1.0e-3 * dy);) {
      if (y > ymax)
        y = ymax;
      l = prenum(y, ndigy, label);
      r2i(x, y, &ix, &iy);
      drawText(ix, iy - 1, 0, 0, AlignRight | AlignVCenter | DontClip, label, l);
      y += ytic;
    }
  }
  frame();
  setPen(pen3);
} 

void KGraph::dir(double ang)
{
  textdir = (float) ang;
}

void KGraph::text(char *str, int ix, int iy, int iorg)
{
  const char fontcode[] = "NHBOSZI", levcode[] = "NUD";
  const int iface[7] = {0, 0, 0, 0, 1, 2, 0};
  const int weight[7] = {QFont::Normal, QFont::Normal, QFont::Bold,
                         QFont::Normal, QFont::Normal, QFont::Normal,
                         QFont::Bold};
  bool italic[7] = {false, false, false, true, false, false, true};
  const double ys[2] = {0.6, -0.4};
  int l, j0, j, ifnt, ilev, idx = 0, idy, tf = 0;
  double dr;
  char *s, *c, *f, code;
  QRect r;

  if ((l = strlen(str))) {
    translate(float(ix), float(iy));
    rotate(-textdir);
    setPen(pen1);
    if (!strchr(str, '#')) {
      switch (iorg) {
        case 1:
          tf = AlignLeft | AlignVCenter | DontClip;
          break;
        case 2:
          tf = AlignCenter | DontClip;
          break;
        case 3:
          tf = AlignRight | AlignVCenter | DontClip;
      }
      drawText(0, 0, 0, 0, tf, str, l);
    } else {
      ix = 0;
      tf = AlignLeft | AlignVCenter | DontClip;
      dr = 0.025 * relsiz * (xmax - xmin) * xscal;
      j0 = (iorg == 1) ? 1 : 0;
      for (j = j0; j < 2; j++) {
        ilev = 0;
        s = str;
        if (j) {
          if (iorg == 2)
            idx >>= 1;
          if (iorg != 1)
            ix = -idx;
        }
        while (*s) {
          l = ((c = strchr(s, '#'))) ? (int) (c - s) : strlen(s);
          if (j == 0) {
            r = boundingRect(0, 0, maxx, maxy, AlignLeft | AlignVCenter, s, l);
            idx += r.width();
          } else {
            idy = (ilev) ? -int(dr * ys[ilev - 1]) : 0;
            drawText(ix, idy, 0, 0, tf, s, l, &r);
            ix += r.width();
          }
          if (c) {
            if (strlen(c) < 3)
              break;
            s = c + 3;
            if ((code = *(++c)) > 96)
              code -= (char) 32;
            if (!(f = strchr(fontcode, code)))
              break;
            ifnt = (int) (f - fontcode);
            if ((code = *(++c)) > 96)
              code -= (char) 32;
            if (!(f = strchr(levcode, code)))
              break;
            QFont fnt = font();
            fnt.setPointSize(max(int(((ilev = (int) (f - levcode)) ? 0.0251 : 0.0355)
                                 * relsiz * (maxx - minx)), 1));
            fnt.setFamily(face[iface[ifnt]]);
            fnt.setWeight(weight[ifnt]);
            fnt.setItalic(italic[ifnt]);
            setFont(fnt);
          } else
            s += l;
        }
        QFont fnt = font();
        fnt.setPointSize(max(int(0.0355 * relsiz * (maxx - minx)), 1));
        fnt.setFamily(face[iface[0]]);
        fnt.setWeight(weight[0]);
        fnt.setItalic(italic[0]);
        setFont(fnt);
      }
    }
    resetXForm();
    setPen(pen3);
  }
}

void KGraph::leth(char *str)
{
  int ix, iy;

  r2i(0.5 * (xmin + xmax),
      ymax - 0.04275 * relsiz * (xmax - xmin) * xscal / yscal, &ix, &iy);
  text(str, ix, iy, 2);
}

void KGraph::letx(char* str, int iex)
{
  letxy(str, iex, 1);
}

void KGraph::lety(char* str, int iex)
{
  letxy(str, iex, 2);
}

void KGraph::symbol(int is)
{
  ksymb = is;
}

void KGraph::circle(double x0, double y0, double r)
{
  int ix, iy, ir;

  ix = int(factor * scalx * x0);
  iy = miny0 + int(factor * scaly * y0);
  ir = int(factor * scalx * r);
  drawEllipse(ix - ir, iy - ir, ir + ir, ir + ir);
}

void KGraph::arcrel(double x, double y, double r, double angst, double angend)
{
  int x0, y0, ir, xstart, ystart;
  double phi;

  r2i(x, y, &x0, &y0);
  ir = int(xscal * fabs(r));
  phi = 1.74532925e-2 * angst;
  xstart = int(ir * cos(phi));
  ystart = int(ir * sin(phi));
  if (r > 0.0) {
    x0 -= xstart;
    y0 += ystart;
  }
  drawArc(x0 - ir, y0 - ir, ir + ir, ir + ir,
          int(16.0 * angst), int(16.0 * angend));
}

void KGraph::plarry(double *x, double *y, double fx, double fy, int n)
{
  const int ax[48] = {0, -2, 0, 2, 0, 2, -2, -2, 2, 2, 0, -2, 2, 0,
                      2, -2, 2, -2, 2, 2, 1, -1, -2, -2, -1, 1, 2, 2,
                      2, -2, 0, -2, 2, 0, 0, 0, -2, 2, 2, 0, -2, 0, 0,
                      0, -2, 2, 0, 0},
            ay[48] = {2, 0, -2, 0, 2, 2, 2, -2, -2, 2, 2, -1, -1, 2,
                      2, 2, -2, -2, 2, 1, 2, 2, 1, -1, -2, -2, -1, 1,
                      2, -2, 0, 2, -2, 2, -2, 0, 0, 0, 2, 0, 2, 0, -2,
                      2, 0, 0, 2, -2},
            as[10] = {0, 5, 10, 14, 19, 28, 33, 38, 43, 48};
  int i = 0, j, k, l = 0, r;
  double f;

  if (ksymb == -17)
    return;
  setClipping(true);
  if (ksymb > 0) {
    linetype(ksymb);
    move(fx * x[0], fy * y[0]);
    for (i = 1; i < n; i++)
      draw(fx * x[i], fy * y[i]);
    linetype(1);
  } else {
    k = (-ksymb) % 17;
    QPointArray ap(10);
    if ((k > 10) || (k == 1)) {
      setBrush(pen3.color());
      setPen(NoPen);
    }
    for (i = 0; i < n; i++) {
      f = 0.005 * relsiz * (maxx - minx);
      move(fx * x[i], fy * y[i]);
      switch (k) {
        case 0:
        case 11:
          r = (int) (f + f);
          drawEllipse(kxcur - r, kycur - r, r + r, r + r);
          break;
        case 1:
          r = (int) (0.4 * f);
          drawEllipse(kxcur - r, kycur - r, r + r, r + r);
          break;
        default:
          r = (k < 12) ? k : k - 10;
          for (j = as[r - 2]; j < as[r - 1]; j++) {
            l = j - as[r - 2];
            if (k < 12)
              if (l)
                lineTo(int(kxcur + f * ax[j]), int(kycur - f * ay[j]));
              else
                moveTo(int(kxcur + f * ax[j]), int(kycur - f * ay[j]));
            else
              ap[l] = QPoint(int(kxcur + f * ax[j]), int(kycur - f * ay[j]));
          }
          if (k >= 12)
            drawPolygon(ap, 0, 0, l);
      }
    }
    if ((k > 10) || (k == 1)) {
      setBrush(NoBrush);
      setPen(pen3);
    }
  }
  setClipping(0);
}

void KGraph::plerry(double *x, double *y, double *e, double fx, double fy, int n)
{
  int i, idx, idy, ie;
  double xa, ya;

  setClipping(true);
  idy = int(0.01 * relsiz * (maxx - minx));
  idx = idy >> 1;
  for (i = 0; i < n; i++) {
    ie = int(-fy * e[i] * yscal);
    xa = fx * x[i];
    ya = fy * y[i];
    if ((xa >= xmin) && (xa <= xmax) &&
        (ya >= ymin) && (ya <= ymax) && (ie >= idy)) {
      move(xa, ya);
      moveTo(kxcur, kycur + idy);
      lineTo(kxcur, kycur + ie);
      moveTo(kxcur - idx, kycur + ie);
      lineTo(kxcur + idx, kycur + ie);
      moveTo(kxcur, kycur - idy);
      lineTo(kxcur, kycur - ie);
      moveTo(kxcur - idx, kycur - ie);
      lineTo(kxcur + idx, kycur - ie);
    }
  }
  setClipping(0);
}

void KGraph::plfun(double xfmin, double xfmax, double dx, double fx, double fy,
           double (*fkt)(double, double *), double *p)
{
  int i, n;
  double x;

  if (!dx)
    dx = 0.01 * (xfmax - xfmin);
  if (ksymb > 0) {
    setClipping(true);
    linetype(ksymb);
    x = xfmin;
    move(fx * x, fy * ((*fkt)(x, p)));
    do {
      x += dx;
      if (fabs(x - xfmax) < 0.9 * fabs(dx))
        x = xfmax;
      draw(fx * x, fy * ((*fkt)(x, p)));
    } while (x != xfmax);
    linetype(0);
    setClipping(0);
  } else {
    n = int(fabs((xfmax - xfmin) / dx));
    double *xv = new double [n];
    double *yv = new double [n];
    for (i = 0; i < n; i++) {
      xv[i] = xmin + i * dx;
      yv[i] = ((*fkt)(xv[i], p));
    }
    plarry(xv, yv, fx, fy, n);
    delete yv;
    delete xv;
  }
}

void KGraph::plpfun(double tmin, double tmax, double dt, double fx, double fy,
                    double (*fktx)(double, double *), double *px,
                    double (*fkty)(double, double *), double *py)
{
  int i, n;
  double t;

  if (!dt)
    dt = 0.01 * (tmax - tmin);
  if (ksymb > 0) {
    setClipping(true);
    linetype(ksymb);
    t = tmin;
    move(fx * ((*fktx)(t, px)), fy * ((*fkty)(t, py)));
    do {
      t += dt;
      if (fabs(t - tmax) < 0.9 * fabs(dt))
        t = tmax;
      draw(fx * ((*fktx)(t, px)), fy * ((*fkty)(t, py)));
    } while (t != tmax);
    linetype(0);
    setClipping(0);
  } else {
    n = int(fabs((tmax - tmin) / dt));
    double *xv = new double [n];
    double *yv = new double [n];
    for (i = 0; i < n; i++) {
      t = tmin + i * dt;
      xv[i] = ((*fktx)(t, px));
      yv[i] = ((*fkty)(t, py));
    }
    plarry(xv, yv, fx, fy, n);
    delete yv;
    delete xv;
  }
}

int KGraph::prenum(double a, int ndig, char *label)
{
  sprintf(label, "%.*f", max(0, ndig), a);
  if (ndig == 0)
    strcat(label, ".");
  return strlen(label);
}

void KGraph::linetype(int is)
{
  const int pat[9][2] = {{8, 8}, {7, 0}, {2, 0}, {1, 0}, {9, 0},
                         {8, 1}, {7, 2}, {4, 4}, {1, 1}};
  int j;
  double f, dsp;

  j = max(0, (is - 1) % 9);
  f = relsiz * (maxx - minx) / (scalx * 320.0);
  dsp = (double) ((16 - pat[j][0] - pat[j][1]) >> 1);
  dash[0] = f * pat[j][0];
  dash[1] = dash[0] + f * dsp;
  dash[2] = dash[1] + f * pat[j][1];
  dash[3] = dash[2] + f * dsp;
}

void KGraph::letxy(char* str, int iex, int iax)
{
  int l, ix, iy;
  double x, y, dx;
  char label[256];

  if (strlen(str) || iex) {
    dx = 0.025 * relsiz * (xmax - xmin);
    if (iax == 2) {
      l = 3 + max(prenum(ymin, ndigy, label), prenum(ymax, ndigy, label));
      x = xmin - (0.75 * l + 0.71) * dx;
      y = 0.5 * (ymin + ymax);
      dir(90.0);
    } else {
      x = 0.5 * (xmin + xmax);
      y = ymin + 3.79 * dx * xscal / yscal;
    }
    r2i(x, y, &ix, &iy);
    if (iex) {
      sprintf(label, "10#nu %d#nn %s", iex, str);
      text(label, ix, iy, 2);
    } else
      text(str, ix, iy, 2);
    if (iax == 2)
      dir(0.0);
  }
}

