/***************************************************************************
                          kgraph.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 1999 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KGRAPH_H
#define KGRAPH_H

#include <qpainter.h>

/**
  * Graphics class. Provides high level plotting functionality.
  * @author Werner Stille
  */
class KGraph : public QPainter
{
public:
  /** Constructor */
  KGraph();
  /**
   * Converts from user units to integer units.
   * @param x x value in user units.
   * @param y y value in user units.
   * @param ix pointer to x value in integer units (output).
   * @param iy pointer to y value in integer units (output).
   */
  void r2i(double x, double y, int *ix, int *iy);
  /**
   * Converts from integer units to centimeters.
   * @param ix x value in integer units.
   * @param iy y value in integer units.
   * @param xcm pointer to x value in centimeters (output).
   * @param ycm pointer to y value in centimeters (output).
   * @return true fr point outside of data window.
   */
  int i2cm(int ix, int iy, double *xcm, double *ycm);
  /**
   * Converts from integer units to user units.
   * @param ix x value in integer units.
   * @param iy y value in integer units.
   * @param fxn normalization factor for x values.
   * @param fyn normalization factor for y values.
   * @param xr pointer to x value in user units (output).
   * @param yr pointer to y value in user units (output).
   */
  void i2r(int ix, int iy, double fxn, double fyn, double *xr, double *yr);
  /**
   * Converts from centimeters to integer units.
   * @param xcm x value in centimeters.
   * @param ycm y value in centimeters.
   * @param fxn normalization factor for x values.
   * @param fyn normalization factor for y values.
   * @param ix pointer to x value in integer units (output).
   * @param iy pointer to y value in integer units (output).
   */
  void cm2r(double xcm, double ycm, double fxn, double fyn,
       double *xr, double *yr);
  /**
   * Sets current (logical) position.
   * @param x x value in user units.
   * @param y y value in user units.
   */
  void move(double x, double y);
  /**
   * Draws line from current to given position.
   * @param x x value in user units.
   * @param y y value in user units.
   */
  void draw(double x, double y);
  /**
   * Initializes plot.
   * @param pd pointer to QPaintdevice.
   * @param offLeft scroll offset from left margin.
   * @param offTop scroll offset from top margin.
   */
  void plInit(const QPaintDevice *pd, int offLeft = 0, int offTop = 0);
  /**
   * (Re)sets page size and resets scaling to centimeters.
   * @param x page width in cm.
   * @param y page height in cm.
   */
  void format(double x, double y);
  /**
   * Sets color for frame and labels.
   * @param icol1 QColor.
   */
  void setColFrame(const QColor & icol1);
  /**
   * Sets color for grids and tics.
   * @param icol2 QColor.
   */
  void setColGrid(const QColor & icol2);
  /**
   * Sets color for data arrays and functions.
   * @param icol2 QColor.
   */
  void setColData(const QColor & icol3);
  /**
   * Sets all colors.
   * @param icol1 QColor for frame and labels.
   * @param icol2 QColor for grids and tics.
   * @param icol3 QColor for data arrays and functions.
   */
  void setCol(const QColor & icol1, const QColor & icol2, const QColor & icol3);
  /**
   * Sets relative size of text, markers and lines.
   * @param r relative size.
   * @return old value of relative size.
   */
  double relsize(double r);
  /**
   * Sets position and size of the data window.
   * @param x0 left page margin in centimeters.
   * @param xl width in centimeters.
   * @param y0 bottom page margin in centimeters.
   * @param yl height in centimeters.
   */
  void Window(double x0, double xl, double y0, double yl);
  /**
   * Scales data window to user units.
   * @param xl x value of left data window margin in user units.
   * @param xr x value of right data window margin in user units.
   * @param yu y value of bottom data window margin in user units.
   * @param yo y value of top window data margin in user units.
   */
  void scale(double xl, double xr, double yu, double yo);
  /**
   * Sets number of digits after decimal point for axis labels.
   * @param nx number of digits for x axis.
   * @param ny number of digits for y axis.
   */
  void setdig(int nx, int ny);
  /** Draws frame around data window. */
  void frame(void);
  /**
   * Draws frame and axes of data window.
   * @param xtic distance between major tics of x axis.
   * @param ytic distance between major tics of y axis.
   * @param intx number of intervals between major tics of x axis.
   * @param inty number of intervals between major tics of y axis.
   * @param mode 0 for frame only, 1 for axes with tics,
   * 2 for axes with tics and labels, 3 for grid, 4 for grid with labels.
   */
  void raster(double xtic, double ytic, int intx, int inty, int mode);
  /**
   * Sets angle of orientation for text plotting.
   * @param ang angle in degrees.
   */
  void dir(double ang);
  /**
   * Plots text.
   * @param str text.
   * @param ix x value in integer units.
   * @param iy y value in integer units.
   * @param iorg 1: left aligned, 2: centered, 3: right aligned.
   */
  void text(char *str, int ix, int iy, int iorg);
  /**
   * Plots heading text.
   * @param str text.
   */
  void leth(char *str);
  /**
   * Plots x axis text.
   * @param str text.
   * @param iex logarithm of normalization factor.
   */
  void letx(char* str, int iex);
  /**
   * Plots y axis text.
   * @param str text.
   * @param iex logarithm of normalization factor.
   */
  void lety(char* str, int iex);
  /**
   * Selects marker or line type.
   * @param is 0 for circles, < 0 for different markers,
   * >0 for different line types.
   */
  void symbol(int is);
  /**
   * Draws a circle.
   * @param x0 x value of center in centimeters.
   * @param y0 y value of center in centimeters.
   * @param r radius in centimeters.
   */
  void circle(double x0, double y0, double r);
  /**
   * Draws an arc.
   * @param x x value in centimeters.
   * @param y y value centimeters.
   * @param r radius in centimeters. r < 0: (x, y) = center,
   * r > 0: (x, y) = start point.
   * @param angst start angle in degrees.
   * @param angend end angle in degrees.
   */
  void arcrel(double x, double y, double r, double angst, double angend);
  /**
   * Draws markers or line given by an array of data points.
   * @param x array of x values in user units.
   * @param y array of y values in user units.
   * @param fx normalization factor for x values.
   * @param fy normalization factor for y values.
   * @param n number of points.
   */
  void plarry(double *x, double *y, double fx, double fy, int n);
  /**
   * Plots error bars for an array of data points.
   * @param x array of x values in user units.
   * @param y array of y values in user units.
   * @param e array of errors of the y values in user units.
   * @param fx normalization factor for x values.
   * @param fy normalization factor for y values.
   * @param n number of points.
   */
  void plerry(double *x, double *y, double *e, double fx, double fy, int n);
  /**
   * Plots ordinary function y = f(x).
   * @param xfmin start x value in user units.
   * @param xfmax end x value in user units.
   * @param dx step size of x values in user units.
   * @param fx normalization factor for x values.
   * @param fy normalization factor for y values.
   * @param fkt pointer to function.
   * @param p array with function parameters.
   */
  void plfun(double xfmin, double xfmax, double dx, double fx, double fy,
             double (*fkt)(double, double *), double *p);
  /**
   * Plots parametric function x = f1(t), y = f2(t).
   * @param tmin start t value.
   * @param tmax end t value.
   * @param dt step size of t values.
   * @param fx normalization factor for x values.
   * @param fy normalization factor for y values.
   * @param fktx pointer to function for x values.
   * @param px array with function parameters for fktx.
   * @param fkty pointer to function for y values.
   * @param py array with function parameters for fkty.
   */
  void plpfun(double tmin, double tmax, double dt, double fx, double fy,
              double (*fktx)(double, double *), double *px,
              double (*fkty)(double, double *), double *py);
  /**
   * Prepares number label.
   * @param a value in user units.
   * @param ndig number of digits after decimal point
   * (-1: no display of decimal point).
   * @param label label (output).
   * @return length of label string.
   */
  int prenum(double a, int ndig, char *label);
  double factor;

protected:
  /**
   * Selects line type.
   * @param is line type (1 for full line).
   */
  void linetype(int is);
  /**
   * Plots axis text.
   * @param str text.
   * @param iex logarithm of normalization factor.
   * @param iax 1 for x axus, 2 for y axis.
   */
  void letxy(char* str, int iex, int iax);

private:
  int minx, maxx, miny, maxy, kxcur, kycur, ndigx, ndigy, ksymb, minx0,
      miny0;
  float textdir;
  double scalx, scaly, xmin, xmax, ymin, ymax, xcur, ycur, dlen,
         dhite, xscal, yscal, relang, relsin, relcos,
         dash[4], dcur, relsiz;
  QPen pen1, pen2, pen3;
};

#endif


