/***************************************************************************
                          kpl.cpp  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sat Apr 24 15:14:00 MEST 1999

    copyright            : (C) 1999 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

// include files for QT
#include <qprinter.h>
#include <qfileinfo.h>
#include <qspinbox.h>
#include <qscrollbar.h>
#include <qimage.h>
#include <qstrlist.h>
#include <qdir.h>

// include files for KDE
#include <kiconloader.h>
#include <kmsgbox.h>
#include <kfm.h>
#include <kfiledialog.h>
#include <kaccel.h>
#include <kprocess.h>
#include <kkeydialog.h>

// application specific includes
#include "kpl.h"
#include "kpldoc.h"
#include "kplview.h"
#include "kplwidget.h"
#include "resource.h"
#include "zoomdlg.h"
#include "refreshdlg.h"
#include "basicdlg.h"
#include "autodlg.h"
#include "itemdlg.h"
#include "autofitdlg.h"

#ifndef min
  #define min(x,y) ((x) < (y) ? (x) : (y))
#endif
#ifndef max
  #define max(x,y) ((x) > (y) ? (x) : (y))
#endif

KplApp::KplApp()
{
  config = kapp->getConfig();
  plot = timerActive = false;
  idTimer = 0;
  tRefresh = 5;
  setMinimumSize(120, 80);
  popupMenu = new QPopupMenu();

  ///////////////////////////////////////////////////////////////////
  // call inits to invoke all other construction parts
  initMenuBar();
  initToolBar();
  initStatusBar();
  initDocument();
  readOptions();
  initView();
  initKeyAccel();

  ///////////////////////////////////////////////////////////////////
  // disable menu and toolbar items at startup
  updWin();

  connect(kapp, SIGNAL(lastWindowClosed()), kapp, SLOT(quit()));
  connect(new KDNDDropZone(this, DndURL),
          SIGNAL(dropAction(KDNDDropZone *)),
          SLOT(slotDropEvent(KDNDDropZone *)));
  connect(doc, SIGNAL(modelChanged()), SLOT(slotUpdate()));
  connect(view->page, SIGNAL(mouseMoved(int, int)), SLOT(updStatus(int, int)));
  connect(view->page, SIGNAL(rightButtonPressed()), SLOT(showPopupMenu()));
  connect(popupMenu, SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
}

KplApp::~KplApp()
{
  delete doc;
  delete view;
  delete key_accel;
  delete popupMenu;
}

void KplApp::openDocumentFile(const char* path)
{
  statusBar()->message(i18n("Opening file..."));
	doc->openDocument(path);
  slotStatusMsg(i18n("Ready."));
}

KplDoc* KplApp::getDocument() const
{
	return doc;
}

void KplApp::initMenuBar()
{
  ///////////////////////////////////////////////////////////////////
  // MENUBAR  

  ///////////////////////////////////////////////////////////////////
  // menuBar entry fileMenu
  recentFilesMenu = new QPopupMenu();
  connect(recentFilesMenu, SIGNAL(activated(int)), SLOT(slotFileOpenRecent(int)));
  fileMenu = new QPopupMenu();
  fileMenu->insertItem(Icon("filenew.xpm"), i18n("&New Plot..."), this,
                       SLOT(slotFileNew()), 0, ID_FILE_NEW);
  fileMenu->insertItem(Icon("fileopen.xpm"), i18n("&Open Plot File..."), this,
                       SLOT(slotFileOpenPlot()), 0, ID_FILE_OPEN_PLOT);
  fileMenu->insertItem(Icon("fileopen.xpm"), i18n("Open &Data File..."), this,
                       SLOT(slotFileOpenData()), 0, ID_FILE_OPEN_DATA);
  fileMenu->insertItem(i18n("Open &Recent"), recentFilesMenu, ID_FILE_OPEN_RECENT);
  fileMenu->insertSeparator();
  fileMenu->insertItem(Icon("filefloppy.xpm") ,i18n("&Save"), this,
                       SLOT(slotFileSave()), 0, ID_FILE_SAVE);
  fileMenu->insertItem(i18n("Save &As..."), this, SLOT(slotFileSaveAs()),
                       0, ID_FILE_SAVE_AS);
  fileMenu->insertItem(i18n("&Close"), this, SLOT(slotFileClose()),
                       0, ID_FILE_CLOSE);
  fileMenu->insertSeparator();
  fileMenu->insertItem(i18n("Display P&lot"), this, SLOT(slotFilePlot()),
                       0, ID_FILE_PLOT);
  fileMenu->insertItem(i18n("PostScript O&utput..."), this, SLOT(slotFilePS()),
                       0, ID_FILE_PS);
  fileMenu->insertSeparator();
  fileMenu->insertItem(Icon("fileprint.xpm"), i18n("&Print..."), this,
                       SLOT(slotFilePrint()), 0, ID_FILE_PRINT);
  fileMenu->insertSeparator();
  fileMenu->insertItem(Icon("mini/kpl.xpm"), i18n("New &Window"), this,
                       SLOT(slotFileNewWindow()), 0, ID_FILE_NEW_WINDOW);
  fileMenu->insertItem(i18n("Close W&indow"), this, SLOT(slotFileCloseWindow()),
                       0, ID_FILE_CLOSE_WINDOW);
  fileMenu->insertSeparator();
  fileMenu->insertItem(i18n("E&xit"), this, SLOT(slotFileQuit()),
                       0, ID_FILE_QUIT);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry viewMenu
  viewMenu = new QPopupMenu();
  viewMenu->setCheckable(true);
  viewMenu->insertItem(Icon("viewmag+.xpm"), i18n("Zoom &In"), this,
                       SLOT(slotViewZoomIn()), 0, ID_VIEW_ZOOM_IN);
  viewMenu->insertItem(Icon("viewmag-.xpm"), i18n("Zoom &Out"), this,
                       SLOT(slotViewZoomOut()), 0, ID_VIEW_ZOOM_OUT);
  viewMenu->insertItem(Icon("viewzoom.xpm"), i18n("&Zoom Factor..."), this,
                       SLOT(slotViewZoom()), 0, ID_VIEW_ZOOM);
  viewMenu->insertSeparator();
  viewMenu->insertItem(Icon("reload.xpm"), i18n("&Reload"), this,
                       SLOT(slotViewReload()), 0, ID_VIEW_RELOAD);
  viewMenu->insertItem(i18n("Reload &Periodically..."), this,
                       SLOT(slotViewAutoReload()), 0, ID_VIEW_AUTO_RELOAD);

  ///////////////////////////////////////////////////////////////////
  // menuBar entry optionsMenu
  optionsMenu = new QPopupMenu();
  optionsMenu->setCheckable(true);
  optionsMenu->insertItem(i18n("Show &Tool Bar"), this,
                          SLOT(slotOptionsToolBar()), 0, ID_OPTIONS_TOOLBAR);
  optionsMenu->insertItem(i18n("Show Stat&us Bar"), this,
                          SLOT(slotOptionsStatusBar()), 0,
                          ID_OPTIONS_STATUSBAR);
  optionsMenu->insertItem(i18n("&Autoplot"), this,
                          SLOT(slotOptionsAutoPlot()), 0, ID_OPTIONS_AUTOPLOT);
  optionsMenu->insertItem(i18n("S&how Function Source"), this,
                          SLOT(slotOptionsShowSource()), 0,
                          ID_OPTIONS_SHOWSOURCE);
  optionsMenu->insertItem(i18n("Calculate &PS Bounding Box"), this,
                          SLOT(slotOptionsBBox()), 0, ID_OPTIONS_BBOX);
  optionsMenu->insertItem(i18n("Unsaved Changes &Warning"), this,
                          SLOT(slotOptionsWarning()), 0, ID_OPTIONS_WARNING);
  optionsMenu->insertItem(i18n("Save Options At &End"), this,
                          SLOT(slotOptionsSaveAtEnd()), 0,
                          ID_OPTIONS_SAVE_AT_END);
  optionsMenu->insertSeparator();
  optionsMenu->insertItem(i18n("Configure &Keys Bindings..."), this,
                          SLOT(slotOptionsKeys()), 0, ID_OPTIONS_KEYS);
  optionsMenu->insertSeparator();
  optionsMenu->insertItem(i18n("Configure &Basics..."), this,
                          SLOT(slotOptionsBasic()), 0, ID_OPTIONS_BASIC);
  optionsMenu->insertItem(i18n("&Configure Autoplot..."), this,
                          SLOT(slotOptionsAuto()), 0, ID_OPTIONS_AUTO);
  optionsMenu->insertItem(i18n("Configure Auto&fit..."), this,
                          SLOT(slotOptionsAutoFit()), 0, ID_OPTIONS_AUTOFIT);
  optionsMenu->insertSeparator();
  optionsMenu->insertItem(i18n("&Save Options"), this,
                          SLOT(slotOptionsSave()), 0, ID_OPTIONS_SAVE);

  ///////////////////////////////////////////////////////////////////
  // MENUBAR CONFIGURATION
  // set menuBar() the current menuBar and the position due to config file
  menuBar()->insertItem(i18n("&File"), fileMenu);
  menuBar()->insertItem(i18n("&Edit"), this, SLOT(slotEditItems()));
  menuBar()->insertItem(i18n("&View"), viewMenu);
  menuBar()->insertItem(i18n("&Options"), optionsMenu);
  menuBar()->insertSeparator();
  menuBar()->insertItem(i18n("&Help"),
                             kapp->getHelpMenu(true, i18n(IDS_APP_ABOUT)));
  connect(menuBar(), SIGNAL(highlighted(int)), SLOT(statusCallback(int)));
}

void KplApp::initToolBar()
{
  ///////////////////////////////////////////////////////////////////
  // TOOLBAR
  // set toolBar() the current toolBar and the position due to config file
  toolBar()->insertButton(Icon("filenew.xpm"), ID_FILE_NEW,
                          SIGNAL(clicked()), this, SLOT(slotFileNew()),
                          true, i18n("New Plot..."));
  toolBar()->insertButton(Icon("fileopen.xpm"), ID_FILE_OPEN_PLOT,
                          SIGNAL(clicked()), this, SLOT(slotFileOpenPlot()),
                          true, i18n("Open Plot File..."));
  toolBar()->insertButton(Icon("fileopen.xpm"), ID_FILE_OPEN_DATA,
                          SIGNAL(clicked()), this, SLOT(slotFileOpenData()),
                          true, i18n("Open Data File..."));
  toolBar()->insertSeparator();
  toolBar()->insertButton(Icon("filefloppy.xpm"), ID_FILE_SAVE,
                          SIGNAL(clicked()), this, SLOT(slotFileSave()),
                          true, i18n("Save"));
  toolBar()->insertSeparator();
  toolBar()->insertButton(Icon("fileprint.xpm"), ID_FILE_PRINT,
                          SIGNAL(clicked()), this, SLOT(slotFilePrint()),
                          true, i18n("Print..."));
  toolBar()->insertSeparator();
  toolBar()->insertButton(Icon("viewmag+.xpm"), ID_VIEW_ZOOM_IN,
                          SIGNAL(clicked()), this, SLOT(slotViewZoomIn()),
                          true, i18n("Zoom In"));
  toolBar()->insertButton(Icon("viewmag-.xpm"), ID_VIEW_ZOOM_OUT,
                          SIGNAL(clicked()), this, SLOT(slotViewZoomOut()),
                          true, i18n("Zoom Out"));
  toolBar()->insertSeparator();
  toolBar()->insertButton(Icon("reload.xpm"), ID_VIEW_RELOAD,
                          SIGNAL(clicked()), this, SLOT(slotViewReload()),
                          true, i18n("Reload"));
  toolBar()->insertSeparator();
  toolBar()->insertButton(Icon("help.xpm"), ID_HELP,
                          SIGNAL(clicked()), kapp, SLOT(appHelpActivated()),
                          true, i18n("Help"));
  connect(toolBar(), SIGNAL(highlighted(int, bool)), SLOT(statusCallback(int)));
}

void KplApp::initStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  //STATUSBAR
  statusBar()->setInsertOrder(KStatusBar::RightToLeft);
  statusBar()->insertItem("  x = -10.00E-99,  y = -15.00E-99", ID_STATUS_VAL);
  statusBar()->insertItem(" 10.00 cm,  15.00 cm,", ID_STATUS_POS);
  statusBar()->insertItem(i18n("Ready."), ID_STATUS_MSG);
  statusBar()->changeItem("", ID_STATUS_VAL);
  statusBar()->changeItem("", ID_STATUS_POS);
}

void KplApp::initKeyAccel()
{
  key_accel = new KAccel(this);

  // fileMenu key accelerators
  CONNECT_ACCEL(KAccel::New, slotFileNew(), fileMenu, ID_FILE_NEW);
  CONNECT_ACCEL(KAccel::Open, slotFileOpenPlot(), fileMenu, ID_FILE_OPEN_PLOT);
  CONNECT_ACCEL(KAccel::Save, slotFileSave(), fileMenu, ID_FILE_SAVE);
  CONNECT_ACCEL(KAccel::Close, slotFileClose(), fileMenu, ID_FILE_CLOSE);
  CONNECT_ACCEL(KAccel::Print, slotFilePrint(), fileMenu, ID_FILE_PRINT);
  CONNECT_ACCEL(KAccel::Quit, slotFileQuit(), fileMenu, ID_FILE_QUIT);

  CONNECT_KEY(Key_F5, "Reload", this, slotViewReload());
  CONNECT_KEY(Key_Left, "Scroll Left", view, slotScrollLeft());
  CONNECT_KEY(Key_Right, "Scroll Right", view, slotScrollRight());
  CONNECT_KEY(Key_Up, "Scroll Up", view, slotScrollUp());
  CONNECT_KEY(Key_Down, "Scroll Down", view, slotScrollDown());
  CONNECT_KEY(Key_PageUp, "Scroll Page Up", view, slotScrollPageUp());
  CONNECT_KEY(Key_PageDown, "Scroll Page Down", view, slotScrollPageDown());
  CONNECT_KEY(CTRL + Key_PageUp, "Scroll Page Left", view, slotScrollPageLeft());
  CONNECT_KEY(CTRL + Key_PageDown, "Scroll Page Right", view, slotScrollPageRight());
  CONNECT_KEY(CTRL + Key_Home, "Scroll to Left Margin", view, slotScrollHomeLeft());
  CONNECT_KEY(Key_Home, "Scroll to Bottom Margin", view, slotScrollHomeDown());
  CONNECT_KEY(CTRL + Key_End, "Scroll to Right Margin", view, slotScrollEndRight());
  CONNECT_KEY(Key_End, "Scroll to Top Margin", view, slotScrollEndUp());
  key_accel->readSettings();	

  // viewMenu key accelerators
  key_accel->changeMenuAccel(viewMenu, ID_VIEW_RELOAD, "Reload");
}

void KplApp::initDocument()
{
  doc = new KplDoc(this);
  doc->newDocument();
}

void KplApp::initView()
{
  ////////////////////////////////////////////////////////////////////
  // create the main widget here that is managed by KTMainWindow's view-region and
  // connect the widget to your document to display document contents.

  view = new KplView(this, fz);
  setView(view);
}

void KplApp::saveOptions(bool saveAll)
{
  int i;
  QString s;
  QStrList s1, s2, s3;

  config->setGroup(kapp->appName());
	config->writeEntry("RecentFiles", recentFiles);
	if (saveAll) {
    config->writeEntry("ShowToolbar", toolBar()->isVisible());
    config->writeEntry("ShowStatusbar", statusBar()->isVisible());
    config->writeEntry("MenuBarPos", (int) menuBar()->menuBarPos());
    config->writeEntry("ToolBar_Pos", (int) toolBar()->barPos());
    config->writeEntry("Directory", doc->currentDir());
    config->writeEntry("Width", width());
    config->writeEntry("Height", height());
    config->writeEntry("AutoPlot", optionsMenu->isItemChecked(ID_OPTIONS_AUTOPLOT));
    config->writeEntry("ShowSource", optionsMenu->isItemChecked(ID_OPTIONS_SHOWSOURCE));
    config->writeEntry("BoundingBox", optionsMenu->isItemChecked(ID_OPTIONS_BBOX));
    config->writeEntry("Warning", optionsMenu->isItemChecked(ID_OPTIONS_WARNING));
    config->writeEntry("SaveAtEnd", optionsMenu->isItemChecked(ID_OPTIONS_SAVE_AT_END));
    config->writeEntry("AutoIndexX", doc->aut.ixAuto);
    config->writeEntry("AutoIndexY", doc->aut.iyAuto);
    config->writeEntry("AutoIndexE", doc->aut.ieAuto);
    config->writeEntry("AutoNorm", doc->aut.autoNorm);
    config->writeEntry("AutoErrBar", doc->aut.autoErr);
    config->writeEntry("AutoGrid", doc->aut.autoGrid);
    config->writeEntry("AutoPathHeader", doc->aut.autohPath);
    config->writeEntry("AutoFit", doc->aut.autoFit);
    config->writeEntry("AutoSymbol", doc->aut.autoSymb);
    config->writeEntry("AutoX0", doc->aut.x0Auto);
    config->writeEntry("AutoY0", doc->aut.y0Auto);
    config->writeEntry("AutoXl", doc->aut.xlAuto);
    config->writeEntry("AutoYl", doc->aut.ylAuto);
    config->writeEntry("FitErrorColumn", doc->aut.fitErrCol);
    config->writeEntry("FitShowDialog", doc->aut.fitShowDlg);
    config->writeEntry("FitSaveParam", doc->aut.fitSavePar);
    config->writeEntry("FitFollow", doc->aut.fitFollow);
    config->writeEntry("FitMaxIterations", doc->aut.fitMaxIt);
    config->writeEntry("FitTolerance", doc->aut.fitTol);
    config->writeEntry("FitSymbol", doc->aut.fitSymb);
    config->writeEntry("FitColor", doc->aut.fitColor);
    config->writeEntry("FitPath", doc->aut.fitPath);
    config->writeEntry("FitName", doc->aut.fitName);
    config->writeEntry("ErrorModelPath", doc->aut.errModPath);
    config->writeEntry("ErrorModelName", doc->aut.errModName);
    for (i = 0; i < NPMAX; i++) {
      s1.insert(i, s.setNum((int) doc->aut.fitPar[i]));
      s2.insert(i, s.setNum(doc->aut.pFit[i]));
      s3.insert(i, s.setNum(doc->aut.pErrMod[i]));
    }
    config->writeEntry("FitEnableParam", s1, ' ');
    config->writeEntry("FitParam", s2, ' ');
    config->writeEntry("ErrorModelParam", s3, ' ');
    config->writeEntry("FormatX", doc->bas.xf);
    config->writeEntry("FormatY", doc->bas.yf);
    config->writeEntry("ColorFrame", doc->bas.colFrame);
    config->writeEntry("ColorGrid", doc->bas.colGrid);
    config->writeEntry("ColorData", doc->bas.colData);
    config->writeEntry("Zoom", view->fZoom);
    config->writeEntry("AutoLetX", doc->aut.sxAuto);
    config->writeEntry("AutoLetY", doc->aut.syAuto);
    config->writeEntry("AutoLetH", doc->aut.shAuto);
  }
  config->sync();
  key_accel->writeSettings();	
}

void KplApp::readOptions()
{
  ///////////////////////////////////////////////////////////////////
  // read the config file entries
  bool b;
  unsigned int i;
  char s1[256];
  QString s, s2;

  config->setGroup(kapp->appName());
  config->setDollarExpansion();
  mainWidth = config->readNumEntry("Width", 640);
  mainHeight = config->readNumEntry("Height", 512);
  resize(mainWidth, mainHeight);
	recentFiles.setAutoDelete(true);
	config->readListEntry("RecentFiles", recentFiles);
	for (i = 0; i < recentFiles.count(); i++)
    recentFilesMenu->insertItem(recentFiles.at(i));
  b = config->readBoolEntry("ShowToolbar", true);
  if (!b)
    enableToolBar(KToolBar::Hide);
  optionsMenu->setItemChecked(ID_OPTIONS_TOOLBAR, b);
  b = config->readBoolEntry("ShowStatusbar", true);
  optionsMenu->setItemChecked(ID_OPTIONS_STATUSBAR, b);
  if (!b)
    enableStatusBar(KStatusBar::Hide);
  menuBar()->setMenuBarPos((KMenuBar::menuPosition) config->readNumEntry("MenuBarPos",
                                                                         KMenuBar::Top));
  toolBar()->setBarPos((KToolBar::BarPosition) config->readNumEntry("ToolBar_Pos",
                                                                    KToolBar::Top));
  doc->setCurrentDir(config->readEntry("Directory", kapp->kde_datadir() + "/" +
                                       kapp->appName()));
  doc->aut.showSource = config->readBoolEntry("ShowSource", false);
  optionsMenu->setItemChecked(ID_OPTIONS_SHOWSOURCE, doc->aut.showSource);
  optionsMenu->setItemChecked(ID_OPTIONS_AUTOPLOT, config->readBoolEntry("AutoPlot", true));
  optionsMenu->setItemChecked(ID_OPTIONS_BBOX, config->readBoolEntry("BoundingBox"));
  optionsMenu->setItemChecked(ID_OPTIONS_WARNING, config->readBoolEntry("Warning", true));
  optionsMenu->setItemChecked(ID_OPTIONS_SAVE_AT_END, config->readBoolEntry("SaveAtEnd"));
  doc->aut.ixAuto = config->readNumEntry("AutoIndexX");
  doc->aut.iyAuto = config->readNumEntry("AutoIndexY", 1);
  doc->aut.ieAuto = config->readNumEntry("AutoIndexE", 2);
  doc->aut.autoNorm = config->readBoolEntry("AutoNorm", true);
  doc->aut.autoErr = config->readBoolEntry("AutoErrBar");
  doc->aut.autoGrid = config->readNumEntry("AutoGrid", 2);
  doc->aut.autohPath = config->readBoolEntry("AutoPathHeader", true);
  doc->aut.autoFit = config->readBoolEntry("AutoFit");
  doc->aut.autoSymb = config->readNumEntry("AutoSymbol", 1);
  doc->aut.x0Auto = config->readDoubleNumEntry("AutoX0", 4.0);
  doc->aut.y0Auto = config->readDoubleNumEntry("AutoY0", 3.0);
  doc->aut.xlAuto = config->readDoubleNumEntry("AutoXl", 15.0);
  doc->aut.ylAuto = config->readDoubleNumEntry("AutoYl", 10.0);
  doc->aut.fitErrCol = config->readBoolEntry("FitErrorColumn");
  doc->aut.fitShowDlg = config->readBoolEntry("FitShowDialog");
  doc->aut.fitSavePar = config->readBoolEntry("FitSaveParam");
  doc->aut.fitFollow = config->readBoolEntry("FitFollow");
  doc->aut.fitMaxIt = config->readNumEntry("FitMaxIterations", 100);
  doc->aut.fitTol = config->readDoubleNumEntry("FitTolerance", 1e-14);
  doc->aut.fitSymb = config->readNumEntry("FitSymbol", 1);
  doc->aut.fitColor = config->readLongNumEntry("FitColor", black.rgb());
  strcpy(doc->aut.fitPath, config->readEntry("FitPath", kapp->kde_datadir() +
                                             "/" + kapp->appName() +
                                             "/fkt.so"));
  strcpy(doc->aut.fitName, config->readEntry("FitName", "polynom"));
  strcpy(doc->aut.errModPath, config->readEntry("ErrorModelPath",
                                                kapp->kde_datadir() +
                                                "/" + kapp->appName() +
                                                "/fkt.so"));
  strcpy(doc->aut.errModName, config->readEntry("ErrorModelName", "polynom"));
  strcpy(s1, config->readEntry("FitEnableParam", "0"));
  doc->aut.fitPar[0] = ((s2 = strtok(s1, " "))) ? s2.toInt() : false;
  for (i = 1; i < NPMAX; i++)
    doc->aut.fitPar[i] = ((s2 = strtok(0, " "))) ? s2.toInt() : false;
  strcpy(s1, config->readEntry("FitParam", "0"));
  doc->aut.pFit[0] = ((s2 = strtok(s1, " "))) ? s2.toDouble() : 0.0;
  for (i = 1; i < NPMAX; i++)
    doc->aut.pFit[i] = ((s2 = strtok(0, " "))) ? s2.toDouble() : 0.0;
  strcpy(s1, config->readEntry("ErrorModelParam", "1.e-14 0.001"));
  doc->aut.pErrMod[0] = ((s2 = strtok(s1, " "))) ? s2.toDouble() : 0.0;
  for (i = 1; i < NPMAX; i++)
    doc->aut.pErrMod[i] = ((s2 = strtok(0, " "))) ? s2.toDouble() : 0.0;
  doc->bas.xf = config->readDoubleNumEntry("FormatX", 21.0);
  doc->bas.yf = config->readDoubleNumEntry("FormatY", 29.7);
  doc->bas.colFrame = config->readLongNumEntry("ColorFrame");
  doc->bas.colGrid = config->readLongNumEntry("ColorGrid");
  doc->bas.colData = config->readLongNumEntry("ColorData");
  fz = config->readDoubleNumEntry("Zoom", 1.0);
  config->setDollarExpansion(false);
  strcpy(doc->aut.sxAuto, config->readEntry("AutoLetX", "x"));
  strcpy(doc->aut.syAuto, config->readEntry("AutoLetY", "y"));
  strcpy(doc->aut.shAuto, config->readEntry("AutoLetH", ""));
 }

void KplApp::enableCommand(int id, bool enable)
{
  ///////////////////////////////////////////////////////////////////
  // enable menu and toolbar functions by their ID's
  menuBar()->setItemEnabled(id, enable);
  toolBar()->setItemEnabled(id, enable);
}

void KplApp::toggleOption(int id)
{
  ///////////////////////////////////////////////////////////////////
  // toggles boolean option
  optionsMenu->setItemChecked(id, !optionsMenu->isItemChecked(id));
  slotStatusMsg(i18n("Ready."));
}

void KplApp::openRead(fileTypes type)
{
  if (optionsMenu->isItemChecked(ID_OPTIONS_WARNING))
    if (!querySave())
      return;
  QString u = KFileDialog::getOpenFileURL(doc->currentDir(), type == DATAFILE ?
                                          "*.dat" : "*.plo");
  if (!u.isEmpty())
    doc->openDocument(u, type);
}

bool KplApp::save(int id)
{
  bool success = false;
  KURL *u = 0;
  QString s;

  if ((doc->currentFile() != i18n("Untitled")) && (id == ID_FILE_SAVE) &&
      (doc->fType == PLOTFILE)) {
    u = new KURL(doc->currentFile());
    if (u->isLocalFile())
      success = doc->saveDocument(doc->currentFile());
    else
      success = doc->saveDocument(QDir::homeDirPath() + "/kpltmp.tmp");
  } else {
    if (getWriteURL(s, "*.plo"))
      success = doc->saveDocument(s);
    if (success) {
      u = new KURL(s);
      if (u->isLocalFile()) {
        QFileInfo fi(s);
        doc->setCurrentDir(fi.dirPath(true));
        doc->setCurrentFile(fi.absFilePath());
      } else {
        doc->setCurrentDir(u->directoryURL());
        doc->setCurrentFile(u->url());
      }
      doc->fType = PLOTFILE;
    }
  }
  if (success) {
    if (!u->isLocalFile()) {
      doc->kfm = new KFM;
      doc->connectKFM();
      doc->kfm->copy(QDir::homeDirPath() + "/kpltmp.tmp",
                     doc->currentFile());
    }
    updWin();
  }
  if (u)
    delete u;
  return success;
}

bool KplApp::getWriteURL(QString &url, const char *filter)
{
  url = KFileDialog::getSaveFileURL(doc->currentDir(), filter);
  if (url.isEmpty())
    return false;
  KURL u = KURL(url);
  if (u.isLocalFile()) {
    url = u.path();
    QFileInfo fi(url);
    if (fi.exists())
      if (fi.isWritable())
        return (KMsgBox::yesNo(this, i18n("Warning"),
                               i18n("A file with this name exists already.\n"
                                    "Do you want to overwrite it?"),
                               KMsgBox::EXCLAMATION) == 1);
      else {
        KMsgBox::message(this, i18n("Sorry:"),
                         "Writing to this file not possible.", 2);
        return false;
      }
  }
  return true;
}

bool KplApp::querySave(void)
{
  if (doc->isModified())
    do {
      int i = KMsgBox::yesNoCancel(this, i18n("Warning"),
                                   i18n("The current plot has been changed.\n"
                                        "Save file?"),
                                   KMsgBox::EXCLAMATION);
      if (i == 3)
        return false;
      if (i == 2)
        break;
    } while (!save(ID_FILE_SAVE));
  return true;
}

void KplApp::addRecentFile(const char* path)
{
  int i;

  if (recentFiles.find(path) == -1) {
    if (recentFiles.count() < 5)
      recentFiles.insert(0, path);
    else {
      recentFiles.remove(4);
      recentFiles.insert(0, path);
    }
    recentFilesMenu->clear();
    for (i = 0; i < (int) recentFiles.count(); i++)
      recentFilesMenu->insertItem(recentFiles.at(i));
	}
}

bool KplApp::queryExit()
{
  if (!querySave())
    return false;
  saveOptions(optionsMenu->isItemChecked(ID_OPTIONS_SAVE_AT_END));
  return true;
}

void KplApp::saveProperties(KConfig *)
{
  if (doc->currentFile() == i18n("Untitled") && !doc->isModified())
    return;
  QString filename = doc->currentFile();
  if (filename == i18n("Untitled"))
    filename = QDir::homeDirPath() + "/" + i18n("Untitled");
  config->writeEntry("filename", filename);
  config->writeEntry("modified", doc->isModified());
  if (doc->isModified())
    doc->saveDocument(kapp->tempSaveName(filename));
}

void KplApp::readProperties(KConfig *)
{
  QString filename = config->readEntry("filename", "");
  bool modified = config->readBoolEntry("modified", false);
  if (modified) {
    bool b_canRecover;
    QString tempname = kapp->checkRecoverFile(filename, b_canRecover);
    if (b_canRecover) {
      doc->openDocument(tempname);
      QFile::remove(tempname);
      doc->setModified();
      if (filename == (QDir::homeDirPath() + "/" + i18n("Untitled")))
        doc->setCurrentFile(i18n("Untitled"));
      else {
        KURL u = KURL(filename);
        if (u.isLocalFile()) {
          QFileInfo fi(filename);
          doc->setCurrentFile(fi.absFilePath());
          doc->setCurrentDir(fi.dirPath(true));
        } else {
          doc->setCurrentDir(u.directoryURL());
          doc->setCurrentFile(u.url());
        }
      }
    }
  } else
    if (!filename.isEmpty()) {
      doc->openDocument(filename);
      KURL u = KURL(filename);
      if (u.isLocalFile()) {
        QFileInfo fi(filename);
        doc->setCurrentFile(fi.absFilePath());
        doc->setCurrentDir(fi.dirPath(true));
      } else {
        doc->setCurrentDir(u.directoryURL());
        doc->setCurrentFile(u.url());
      }
    }
  updWin();
}		

/////////////////////////////////////////////////////////////////////
// EVENT HANDLER IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KplApp::closeEvent(QCloseEvent *e)
{
  ///////////////////////////////////////////////////////////////////
  // exits the Application
  if (this->queryExit()) {
    saveOptions(optionsMenu->isItemChecked(ID_OPTIONS_SAVE_AT_END));
    e->accept();
  } else {
    slotStatusMsg(i18n("Ready."));
    e->ignore();
  }
}

void KplApp::timerEvent(QTimerEvent *e)
{
  if (e->timerId() == idTimer)
    slotViewReload();
}

/////////////////////////////////////////////////////////////////////
// SLOT IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

void KplApp::statusCallback(int id) {
  switch (id) {
    ON_STATUS_MSG(ID_FILE_NEW_WINDOW, i18n("Opens a new application window"))
    ON_STATUS_MSG(ID_FILE_NEW, i18n("Creates a new plot"))
    ON_STATUS_MSG(ID_FILE_OPEN_PLOT, i18n("Opens an existing plot file"))
    ON_STATUS_MSG(ID_FILE_OPEN_DATA, i18n("Opens an existing data file"))
    ON_STATUS_MSG(ID_FILE_SAVE, i18n("Saves the current plot"))
    ON_STATUS_MSG(ID_FILE_SAVE_AS, i18n("Saves the plot under a "
                                        "different name"))
    ON_STATUS_MSG(ID_FILE_CLOSE, i18n("Closes the current plot"))
    ON_STATUS_MSG(ID_FILE_PLOT, i18n("Displays the current plot"))
    ON_STATUS_MSG(ID_FILE_PS, i18n("Prints to PostScript file"))
    ON_STATUS_MSG(ID_FILE_PRINT, i18n("Prints the current plot"))
    ON_STATUS_MSG(ID_FILE_CLOSE_WINDOW, i18n("Closes the current window"))
    ON_STATUS_MSG(ID_FILE_QUIT, i18n("Exits the program"))

    ON_STATUS_MSG(ID_EDIT, i18n("Edits plot items"))

    ON_STATUS_MSG(ID_VIEW_ZOOM_IN, i18n("Increases magnification by 20 %"))
    ON_STATUS_MSG(ID_VIEW_ZOOM_OUT, i18n("Decreases magnification by 20 %"))
    ON_STATUS_MSG(ID_VIEW_ZOOM, i18n("Sets magnification"))
    ON_STATUS_MSG(ID_VIEW_RELOAD, i18n("Reloads file"))
    ON_STATUS_MSG(ID_VIEW_AUTO_RELOAD, i18n("Reloads file periodically"))

    ON_STATUS_MSG(ID_OPTIONS_TOOLBAR, i18n("Enables / disables the toolbar"))
    ON_STATUS_MSG(ID_OPTIONS_STATUSBAR, i18n("Enables / disables "
                                             "the statusbar"))
    ON_STATUS_MSG(ID_OPTIONS_AUTOPLOT, i18n("Enables / disables "
                                            "automatic plotting"))
    ON_STATUS_MSG(ID_OPTIONS_SHOWSOURCE, i18n("Enables / disables "
                                              "display of function source"))
    ON_STATUS_MSG(ID_OPTIONS_BBOX, i18n("Enables / disables "
                                        "calculation of PS bounding box"))
    ON_STATUS_MSG(ID_OPTIONS_WARNING, i18n("Enables / disables warning "
                                           "on unsaved changes"))
    ON_STATUS_MSG(ID_OPTIONS_SAVE_AT_END, i18n("Enables / disables saving "
                                               "options at end"))
    ON_STATUS_MSG(ID_OPTIONS_KEYS, i18n("Displays key bindings dialog"))
    ON_STATUS_MSG(ID_OPTIONS_BASIC, i18n("Displays basic settings dialog"))
    ON_STATUS_MSG(ID_OPTIONS_AUTO, i18n("Displays autoplot settings dialog"))
    ON_STATUS_MSG(ID_OPTIONS_AUTOFIT, i18n("Displays autofit settings dialog"))
    ON_STATUS_MSG(ID_OPTIONS_SAVE, i18n("Saves options"))
    ON_STATUS_MSG(ID_HELP, i18n("Help"))
  }
}

void KplApp::slotFileNew()
{
  if (querySave()) {
    doc->newDocument();
    updWin();
    view->eraseWidget();
    slotEditItems();
  }
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFileOpenPlot()
{
  statusBar()->message(i18n("Opening file..."));
  openRead(PLOTFILE);
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFileOpenData()
{
  statusBar()->message(i18n("Opening file..."));
  openRead(DATAFILE);
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFileOpenRecent(int id)
{
  statusBar()->message(i18n("Opening file..."));
  if (optionsMenu->isItemChecked(ID_OPTIONS_WARNING))
    if (!querySave())
      return;
  doc->openDocument(recentFiles.at(id));
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFileSave()
{
  statusBar()->message(i18n("Saving file..."));
  save(ID_FILE_SAVE);
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFileSaveAs()
{
  statusBar()->message(i18n("Saving file under different name..."));
  save(ID_FILE_SAVE_AS);
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFileClose()
{
  statusBar()->message(i18n("Closing file..."));
  if (querySave()) {
    doc->newDocument();
    updWin();
    view->eraseWidget();
  }
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFilePlot()
{
  statusBar()->message(i18n("Plotting..."));
  plot = doc->nItems;
  slotPlotCond();
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotPlotCond()
{
  if (plot)
    view->paintWidget();
  else
    view->eraseWidget();
}

void KplApp::slotFilePS()
{
  statusBar()->message(i18n("Generating PostScript file..."));
  QString s;
  FILE *f = 0;
  if (getWriteURL(s, "*.ps *.eps"))
    if ((f = fopen(s, "w"))) {
      view->page->setCursor(waitCursor);
      view->doPS(f);
      if (optionsMenu->isItemChecked(ID_OPTIONS_BBOX)) {
        KProcess kp;
        QString s2;
        kp << "gs" << "-dNOPAUSE" << "-dBATCH" << "-q" << "-sDEVICE=pbmraw"
           << ("-sOutputFile=" + QDir::homeDirPath() + "/kpltmp.pbm") << "-r72"
           << s2.sprintf("-g%ix%i", int(28.34645669 * doc->bas.xf + 0.5),
                         int(28.34645669 * doc->bas.xf + 0.5))
           << s;
        kp.start(KProcess::Block);
        QImage image;
        if (image.load(QDir::homeDirPath() + "/kpltmp.pbm", "PBMRAW")) {
          int x, y, xl = 0, yo = 0;
          int xr = image.width();
          int yu = image.height();
          for (y = 0; y < image.height(); y++)
            for (x = 0; x < image.width(); x++)
              if (!(image.pixel(x, y) & 0xffffff)) {
                yo = y;
                xl = xr = x;
                goto yo_found;
              }
  yo_found:
          for (y = image.height() - 1; y > yo; y--)
            for (x = 0; x < image.width(); x++)
              if (!(image.pixel(x, y) & 0xffffff)) {
                yu = y;
                xl = min(xl, x);
                xr = max(xr, x);
                goto yu_found;
              }
  yu_found:
          for (y = yo; y <= yu; y++){
            for (x = 0; x < xl; x++)
              if (!(image.pixel(x, y) & 0xffffff)) {
                xl = x;
                break;
              }
            for (x = image.width() - 1; x > xr; x--)
              if (!(image.pixel(x, y) & 0xffffff)) {
                xr = x;
                break;
              }
          }
          xl = max(xl - 2, 0);
          xr = min(xr + 2, image.width() - 1);
          yu = max(image.height() - yu - 2, 0);
          yo = min(image.height() - yo + 2, image.height() - 1);
          remove(QDir::homeDirPath() + "/kpltmp.pbm");
          f = fopen(s, "r+");
          fprintf(f, "%%!PS-Adobe-2.0 EPSF-1.2\n%%%%BoundingBox: %i %i %i %i      ",
                  xl, yu, xr, yo);
          fclose(f);
        } else
          KMsgBox::message(this, i18n("Error"),
                           i18n("Bitmap file for bounding box calculation not loaded!"),
                           KMsgBox::EXCLAMATION);
      }
      QFileInfo fi(s);
      doc->setCurrentDir(fi.dirPath(true));
      view->page->setCursor(crossCursor);
    }
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFilePrint()
{
  statusBar()->message(i18n("Printing..."));
  QPrinter printer;
  if (printer.setup(this))
    view->print(&printer);
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFileNewWindow()
{
  (new KplApp)->show();
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotFileCloseWindow()
{
  close();
}

void KplApp::slotFileQuit()
{
  ///////////////////////////////////////////////////////////////////
  // exits the Application
  if(this->queryExit()) {
    saveOptions(optionsMenu->isItemChecked(ID_OPTIONS_SAVE_AT_END));
    KTMainWindow::deleteAll();
    kapp->quit();
  } else
    slotStatusMsg(i18n("Ready."));
}

void KplApp::slotEditItems()
{
  statusBar()->message(i18n("Editing plot items..."));
  ItemDlg dlg(this, doc);
  connect(&dlg, SIGNAL(displayMessage(const char *)),
           SLOT(slotStatusMsg(const char *)));
  if (dlg.exec()) {
    doc->setModified();
    slotUpdate();
  }
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotViewZoomIn()
{
  ///////////////////////////////////////////////////////////////////
  // set magnification
  view->fZoom = 0.01 * (min(int(100 * view->fZoom + 0.5) + 20, 2000));
  shrink();
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotViewZoomOut()
{
  ///////////////////////////////////////////////////////////////////
  // set magnification
  view->fZoom = 0.01 * (max(int(100 * view->fZoom + 0.5) - 20, 1));
  shrink();
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotViewZoom()
{
  ///////////////////////////////////////////////////////////////////
  // set magnification
  statusBar()->message(i18n("Setting magnification..."));
  ZoomDlg dlg(this);
  dlg.factor->setValue(int(100 * view->fZoom + 0.5));
  if (dlg.exec() == dlg.Accepted) {
    view->fZoom = 0.01 * dlg.factor->value();
    shrink();
  }
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotViewReload()
{
  ///////////////////////////////////////////////////////////////////
  // reload file
  statusBar()->message(i18n("Reloading file..."));
  menuBar()->setItemChecked(ID_OPTIONS_AUTOPLOT, true);
  doc->openDocument(doc->currentFile(), doc->fType);
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotViewAutoReload()
{
  ///////////////////////////////////////////////////////////////////
  // reload file periodically
  statusBar()->message(i18n("Reloading file periodically..."));
  RefreshDlg dlg(this);
  dlg.setWidgets(tRefresh, timerActive);
  if (dlg.exec() == dlg.Accepted) {
    int tRef;
    bool act;
    dlg.getValues(&tRef, &act);
    if (timerActive && ((tRef != tRefresh) || (act != timerActive)))
      killTimer(idTimer);
    if (act && ((tRef != tRefresh) || (act != timerActive))) {
      idTimer = startTimer(1000 * tRef);
      if (!idTimer) {
        KMsgBox::message(this, i18n("Error"),
                         i18n("No timer available!"), KMsgBox::EXCLAMATION);
        act = 0;
      }
    }
    timerActive = act;
    tRefresh = tRef;
  }
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotOptionsToolBar()
{
  ///////////////////////////////////////////////////////////////////
  // turn Toolbar on or off
  toggleOption(ID_OPTIONS_TOOLBAR);
  enableToolBar();
}

void KplApp::slotOptionsStatusBar()
{
  ///////////////////////////////////////////////////////////////////
  //turn Statusbar on or off
  toggleOption(ID_OPTIONS_STATUSBAR);
  enableStatusBar();
}

void KplApp::slotOptionsWarning()
{
  ///////////////////////////////////////////////////////////////////
  //turn unsaved warning plotting on or off
  toggleOption(ID_OPTIONS_WARNING);
}

void KplApp::slotOptionsAutoPlot()
{
  ///////////////////////////////////////////////////////////////////
  //turn automatic plotting on or off
  toggleOption(ID_OPTIONS_AUTOPLOT);
  if (optionsMenu->isItemChecked(ID_OPTIONS_AUTOPLOT))
    slotFilePlot();
}

void KplApp::slotOptionsShowSource()
{
  ///////////////////////////////////////////////////////////////////
  //turn display of function source on or off
  toggleOption(ID_OPTIONS_SHOWSOURCE);
  doc->aut.showSource = optionsMenu->isItemChecked(ID_OPTIONS_SHOWSOURCE);
}

void KplApp::slotOptionsBBox()
{
  ///////////////////////////////////////////////////////////////////
  //turn calculation of PS bounding box on or off
  toggleOption(ID_OPTIONS_BBOX);
}

void KplApp::slotOptionsSaveAtEnd()
{
  ///////////////////////////////////////////////////////////////////
  //turn save-at-end option on or off
  toggleOption(ID_OPTIONS_SAVE_AT_END);
}

void KplApp::slotOptionsKeys()
{
  ///////////////////////////////////////////////////////////////////
  //display key bindings dialog
  statusBar()->message(i18n("Key bindings..."));
  KKeyDialog::configureKeys(key_accel, false);
  key_accel->changeMenuAccel(viewMenu, ID_VIEW_RELOAD, "Reload");
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotOptionsBasic()
{
  ///////////////////////////////////////////////////////////////////
  //display basic settings dialog
  statusBar()->message(i18n("Basic settings..."));
  BasicDlg dlg(this, &doc->bas);
  if (dlg.exec() == dlg.Accepted)
    shrink();
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotOptionsAuto()
{
  ///////////////////////////////////////////////////////////////////
  //display autoplot settings dialog
  statusBar()->message(i18n("Autoplot settings..."));
  AutoDlg dlg(this, doc);
  dlg.exec();
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotOptionsAutoFit()
{
  ///////////////////////////////////////////////////////////////////
  //display autofit settings dialog
  statusBar()->message(i18n("Autofit settings..."));
  AutoFitDlg dlg(this, doc);
  dlg.exec();
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotOptionsSave()
{
  ///////////////////////////////////////////////////////////////////
  //save options
  saveOptions(true);
  slotStatusMsg(i18n("Ready."));
}

void KplApp::slotStatusMsg(const char *text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message permanently
  statusBar()->clear();
  statusBar()->changeItem(text, ID_STATUS_MSG);
}

void KplApp::slotStatusHelpMsg(const char *text)
{
  ///////////////////////////////////////////////////////////////////
  // change status message of whole statusbar temporary (text, msec)
  statusBar()->message(text, 3000);
}

void KplApp::slotDropEvent(KDNDDropZone *zone)
{
  QString u = zone->getURLList().first();
  if (optionsMenu->isItemChecked(ID_OPTIONS_WARNING) ? querySave() : true)
    doc->openDocument(u);
}

void KplApp::slotUpdate()
{
  updWin();
  slotPlotCond();
}

void KplApp::updWin()
{
  QString s;

  if (doc->currentFile() != i18n("Untitled")) {
    setCaption(doc->currentFile() + " - kpl");
    addRecentFile(doc->currentFile());
  } else
    setCaption("kpl");
  enableCommand(ID_FILE_OPEN_RECENT, recentFiles.count());
  enableCommand(ID_FILE_SAVE_AS, doc->nItems);
  enableCommand(ID_FILE_CLOSE, doc->nItems);
  enableCommand(ID_FILE_PLOT, doc->nItems);
  enableCommand(ID_FILE_PS, doc->nItems);
  enableCommand(ID_FILE_PRINT, doc->nItems);
  enableCommand(ID_FILE_SAVE, doc->isModified());
  enableCommand(ID_VIEW_RELOAD, (doc->currentFile() != i18n("Untitled")));
  plot = optionsMenu->isItemChecked(ID_OPTIONS_AUTOPLOT) && doc->nItems;
}

void KplApp::updStatus(int x, int y)
{
  double xcm, ycm, xr, yr;
  QString s;

  if (statusBar()->isVisible()) {
    if ((!view->i2cm(x, y, &xcm, &ycm)) && plot) {
      view->i2r(x, y, view->fxn, view->fyn, &xr, &yr);
      statusBar()->changeItem(s.sprintf("  x = %9.4G, y = %9.4G", xr, yr),
                              ID_STATUS_VAL);
      statusBar()->changeItem(s.sprintf("%6.2f cm, %6.2f cm,", xcm, ycm),
                              ID_STATUS_POS);
    } else {
      statusBar()->changeItem("", ID_STATUS_VAL);
      statusBar()->changeItem(s.sprintf("%6.2f cm, %6.2f cm", xcm, ycm),
                              ID_STATUS_POS);
    }
  }
}

void KplApp::shrink()
{
  view->layout();
  if ((view->xPict < view->scrx) || (view->yPict < view->scry)) {
    int no = 1 + (toolBar()->sizeHint().width() - 1) / width();
    int w = min(view->xPict, view->scrx) - view->scrx + width() + 2;
    int n = 1 + (toolBar()->sizeHint().width() - 1) / w;
    resize(w, min(view->yPict, view->scry) - view->scry + height() +
           menuBar()->heightForWidth(w) - menuBar()->height() +
           (n - no) * (toolBar()->height() / no) + 2);
  } else
    slotPlotCond();
}

void KplApp::showPopupMenu()
{
  popupMenu->clear();
  popupMenu->insertItem(Icon("filenew.xpm"), i18n("&New Plot..."), this,
                        SLOT(slotFileNew()), 0, ID_FILE_NEW);
  popupMenu->insertItem(Icon("fileopen.xpm"), i18n("&Open Plot File..."), this,
                        SLOT(slotFileOpenPlot()), 0, ID_FILE_OPEN_PLOT);
  popupMenu->insertItem(Icon("fileopen.xpm"), i18n("Open &Data File..."), this,
                        SLOT(slotFileOpenData()), 0, ID_FILE_OPEN_DATA);
  if (doc->isModified())
    popupMenu->insertItem(Icon("filefloppy.xpm") ,i18n("&Save"), this,
                          SLOT(slotFileSave()), 0, ID_FILE_SAVE);
  if (doc->nItems) {
    popupMenu->insertItem(i18n("Save &As..."), this, SLOT(slotFileSaveAs()),
                          0, ID_FILE_SAVE_AS);
    popupMenu->insertItem(i18n("PostScript O&utput..."), this, SLOT(slotFilePS()),
                          0, ID_FILE_PS);
    popupMenu->insertItem(Icon("fileprint.xpm"), i18n("&Print..."), this,
                          SLOT(slotFilePrint()), 0, ID_FILE_PRINT);
  }
  popupMenu->insertSeparator();
  popupMenu->insertItem(i18n("&Edit"), this, SLOT(slotEditItems()), 0, ID_EDIT);
  if (doc->nItems) {
    popupMenu->insertSeparator();
    popupMenu->insertItem(Icon("viewmag+.xpm"), i18n("Zoom &In"), this,
                          SLOT(slotViewZoomIn()), 0, ID_VIEW_ZOOM_IN);
    popupMenu->insertItem(Icon("viewmag-.xpm"), i18n("Zoom &Out"), this,
                          SLOT(slotViewZoomOut()), 0, ID_VIEW_ZOOM_OUT);
  }
  if (doc->currentFile() != i18n("Untitled"))
    popupMenu->insertItem(Icon("reload.xpm"), i18n("&Reload"), this,
                          SLOT(slotViewReload()), 0, ID_VIEW_RELOAD);
  popupMenu->exec(QCursor::pos());
}
