/***************************************************************************
                          kpldoc.cpp  -  description                              
                             -------------------                                         

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sat Apr 24 15:14:00 MEST 1999
                                           
    copyright            : (C) 1999 by Werner Stille                         
    email                : stille@uni-freiburg.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <math.h>
#include <dlfcn.h>
#include <qfileinfo.h>
#include <qdir.h>
#include <kapp.h>
#include <kfm.h>
#include <kmsgbox.h>
#include "kpldoc.h"
#include "fitdlg.h"

#ifndef min
  #define min(x,y) ((x) < (y) ? (x) : (y))
#endif
#ifndef max
  #define max(x,y) ((x) > (y) ? (x) : (y))
#endif

const int KplDoc::iSize[6] = {0, sizeof(FRAMEDATA), sizeof(ARRAYDATA),
                              sizeof(FUNDATA), sizeof(PARFUNDATA),
                              sizeof(LEGENDDATA)};
const char KplDoc::itemName[6][11] = {"", "FRAMEITEM", "ARRAYITEM", "FUNITEM",
                                   "PARFUNITEM", "LEGENDITEM"};

KplDoc::KplDoc(QObject *parent) :
 QObject(parent)
{
  nItems = 0;
  fType = UNKNOWN;
  itd = 0;
}

KplDoc::~KplDoc()
{
  freeAllItd();
}

void KplDoc::newDocument()
{
  m_modified = false;
  m_file = i18n("Untitled");
  freeAllItd();
  nyLeg = 0;
}

bool KplDoc::saveDocument(const char* filename)
{
  FILE *f = fopen(filename, "w");
  if (f) {
    writePlo(f);
    m_modified = false;
  } else
    KMsgBox::message((QWidget *) parent(), i18n("Sorry:"),
                     "Writing to this file not possible.",
                     KMsgBox::EXCLAMATION);
  return (f);
}

void KplDoc::setModified(bool modified)
{
  m_modified = modified;
}

bool KplDoc::isModified()
{
  return m_modified;
}

void KplDoc::setCurrentDir(const char *dir)
{
  m_dir = dir;
}

const QString& KplDoc::currentDir() const
{
  return m_dir;
}

void KplDoc::setCurrentFile(const char* file)
{
  m_file = file;
}

const QString& KplDoc::currentFile() const
{
  return m_file;
}

void KplDoc::freeItd(PITEMDATA iti)
{
  switch (iti->ityp) {
    case ARRAYITEM:
      if (((PARRAYDATA) iti->stru)->x)
        freeXCond((PARRAYDATA) iti->stru);
      break;
    case FUNITEM:
      if (((PFUNDATA) iti->stru)->hmod)
        dlclose(((PFUNDATA) iti->stru)->hmod);
      break;
    case PARFUNITEM:
      if (((PPARFUNDATA) iti->stru)->hmodx)
        dlclose(((PPARFUNDATA) iti->stru)->hmodx);
      if (((PPARFUNDATA) iti->stru)->hmody)
        dlclose(((PPARFUNDATA) iti->stru)->hmody);
      break;
    default:
      break;
  }
  free(iti->stru);
}

void KplDoc::freeAllItd()
{
  int i;

  for (i = nItems - 1; i >= 0; i--) {
    freeItd(&itd[i]);
    nItems--;
  }
  if (itd) {
    free(itd);
    itd = 0;
  }
  setModified(false);
}

void KplDoc::freeX(double ***x)
{
  if (*x) {
    free((*x)[0]);
    free(*x);
    *x = 0;
  }
}

void KplDoc::freeXCond(PARRAYDATA ad)
{
  bool del = true;
  int i;

  for (i = 0; i < nItems; i++)
    if ((itd[i].stru != ad) && (itd[i].ityp == ARRAYITEM))
      if (((PARRAYDATA) itd[i].stru)->x == ad->x)
        del = false;
  if (del)
    freeX(&ad->x);
}

int KplDoc::readFile(QString path, double ***x, int *ncols, int *nrows)
{
  const char delim[] = " ,\t";
  int i, j, k, np;
  double *x1, *x2 = 0;
  char s[256], *s1;
  QString fn;
  FILE *f;

  freeX(x);
  *ncols = *nrows = 0;
  KURL u = KURL(path);
  if (u.isLocalFile())
    fn = u.path();
  else
    if (!KFM::download(u.url(), fn))
      return *nrows;
  if ((f = fopen(fn, "r"))) {
    do
      if ((i = (fgets(s, 256, f) != 0)))
        i = (s[0] == '#') ? 1 : 2;
    while (i == 1);
    if (i) {
      s1 = strtok(s, delim);
      while (s1) {
        (*ncols)++;
        s1 = strtok(0, delim);
      }
      if (*ncols) {
        rewind(f);
        np = 0;
        k = 0;
        while (fgets(s, 256, f))
          if (s[0] != '#') {
            (*nrows)++;
            np += *ncols;
            x2 = (double *) realloc(x2, np * sizeof(double));
            x2[k++] = ((s1 = strtok(s, delim))) ? atof(s1) : 0.0;
            for (i = 1; i < *ncols; i++)
              x2[k++] = ((s1 = strtok(0, delim))) ? atof(s1) : 0.0;
          }
        x1 = (double *) malloc(np * sizeof(double));
        *x = (double **) malloc(*ncols * sizeof(double *));
        for (i = 0; i < *ncols; i++)
          (*x)[i] = &x1[i * *nrows];
        k = 0;
        for (j = 0; j < *nrows; j++)
          for (i = 0; i < *ncols; i++)
            (*x)[i][j] = x2[k++];
        free(x2);
      }
    }
    fclose(f);
  }
  if (!u.isLocalFile())
    KFM::removeTempFile(fn);
  return *nrows;
}

void KplDoc::iniStru(PITEMDATA itd, itemTypes ityp)
{
  double dx = aut.xlAuto / 15.0;

  itd->ityp = ityp;
  itd->stru = calloc(1, iSize[ityp]);
  switch (ityp) {
    case FRAMEITEM:
      ((PFRAMEDATA) itd->stru)->xf = bas.xf;
      ((PFRAMEDATA) itd->stru)->yf = bas.yf;
      ((PFRAMEDATA) itd->stru)->colFrame = bas.colFrame;
      ((PFRAMEDATA) itd->stru)->colGrid = bas.colGrid;
      ((PFRAMEDATA) itd->stru)->xw0 = aut.x0Auto;
      ((PFRAMEDATA) itd->stru)->xw = aut.xlAuto;
      ((PFRAMEDATA) itd->stru)->yw0 = aut.y0Auto;
      ((PFRAMEDATA) itd->stru)->yw = aut.ylAuto;
      ((PFRAMEDATA) itd->stru)->xma = aut.xlAuto;
      ((PFRAMEDATA) itd->stru)->yma = aut.ylAuto;
      ((PFRAMEDATA) itd->stru)->xtic = 5.0;
      ((PFRAMEDATA) itd->stru)->ytic = 2.0;
      ((PFRAMEDATA) itd->stru)->relsiz = 1.0;
      ((PFRAMEDATA) itd->stru)->mticx = 5;
      ((PFRAMEDATA) itd->stru)->mticy = 2;
      ((PFRAMEDATA) itd->stru)->ndigx = -1;
      ((PFRAMEDATA) itd->stru)->ndigy = -1;
      ((PFRAMEDATA) itd->stru)->gridmode = aut.autoGrid;
      strcpy(((PFRAMEDATA) itd->stru)->sx, aut.sxAuto);
      strcpy(((PFRAMEDATA) itd->stru)->sy, aut.syAuto);
      strcpy(((PFRAMEDATA) itd->stru)->sh, aut.shAuto);
      break;
    case ARRAYITEM:
      ((PARRAYDATA) itd->stru)->color = bas.colData;
      ((PARRAYDATA) itd->stru)->fx = 1.0;
      ((PARRAYDATA) itd->stru)->fy = 1.0;
      ((PARRAYDATA) itd->stru)->ix = aut.ixAuto;
      ((PARRAYDATA) itd->stru)->iy = aut.iyAuto;
      ((PARRAYDATA) itd->stru)->ie = aut.ieAuto;
      ((PARRAYDATA) itd->stru)->symb = aut.autoSymb;
      ((PARRAYDATA) itd->stru)->errbars = aut.autoErr;
      break;
    case FUNITEM:
      ((PFUNDATA) itd->stru)->color = bas.colData;
      ((PFUNDATA) itd->stru)->fx = 1.0;
      ((PFUNDATA) itd->stru)->fy = 1.0;
      ((PFUNDATA) itd->stru)->symb = aut.autoSymb;
      break;
    case PARFUNITEM:
      ((PPARFUNDATA) itd->stru)->color = bas.colData;
      ((PPARFUNDATA) itd->stru)->fx = 1.0;
      ((PPARFUNDATA) itd->stru)->fy = 1.0;
      ((PPARFUNDATA) itd->stru)->symb = aut.autoSymb;
      break;
    case LEGENDITEM:
      ((PLEGENDDATA) itd->stru)->colsymb = bas.colData;
      ((PLEGENDDATA) itd->stru)->coltext = bas.colFrame;
      ((PLEGENDDATA) itd->stru)->x = aut.x0Auto + dx;
      ((PLEGENDDATA) itd->stru)->y = aut.y0Auto + aut.ylAuto
        - (++nyLeg) * dx;
      ((PLEGENDDATA) itd->stru)->xtext = dx;
      ((PLEGENDDATA) itd->stru)->ytext = -0.18 * dx;
      ((PLEGENDDATA) itd->stru)->relsiz = 1.0;
      ((PLEGENDDATA) itd->stru)->symb = aut.autoSymb;
  }
}

bool KplDoc::autoStruc(PFRAMEDATA fd, PARRAYDATA ad)
{
  if ((ad->x) && (aut.ixAuto < ad->ncols) && (aut.iyAuto < ad->ncols)) {
    if (autoFl(&fd->xmi, &fd->xma, &fd->xtic, &fd->mticx, &fd->ndigx, &fd->iex,
               &ad->fx, ad->x[aut.ixAuto], ad->nrows, aut.autoNorm, 0.2))
      return true;
    if (autoFl(&fd->ymi, &fd->yma, &fd->ytic, &fd->mticy, &fd->ndigy, &fd->iey,
               &ad->fy, ad->x[aut.iyAuto], ad->nrows, aut.autoNorm,
               0.2 * aut.xlAuto / aut.ylAuto))
      return true;
    if (aut.autohPath)
      strncpy(fd->sh, ad->path, sizeof(aut.shAuto) - 1);
    ad->ix = aut.ixAuto;
    ad->iy = aut.iyAuto;
    ad->ie = aut.ieAuto;
    ad->symb = aut.autoSymb;
    ad->istart = 0;
    ad->n = ad->nrows;
    ad->errbars = aut.autoErr & (aut.ieAuto < ad->ncols);
    return false;
  }
  return true;
}

void KplDoc::autoFit(PARRAYDATA ad, PFUNDATA fd)
{
  strcpy(fd->path, aut.fitPath);
  strcpy(fd->name, aut.fitName);
  memcpy(fd->p, aut.pFit, NPMAX * sizeof(double));
  fd->symb = aut.fitSymb;
  fd->color = aut.fitColor;
  fd->xfmin = ad->x[ad->ix][ad->istart];
  fd->xfmax = ad->x[ad->ix][ad->istart + ad->n - 1];
  fd->fx = ad->fx;
  fd->fy = ad->fy;
  if (getFun(fd->path, fd->name, &fd->hmod, &fd->fkt))
    return;
  int mode = 0;
  if (aut.fitShowDlg)
    mode |= SHOWDLG;
  if (aut.fitSavePar)
    mode |= SAVEPAR;
  if (aut.fitFollow)
    mode |= FOLLOW;
  FitDlg dlg((QWidget *) parent(), this, ad, fd, mode);
  dlg.slotFit();
  if (aut.fitShowDlg) {
    connect(&dlg, SIGNAL(modelChanged()), this, SIGNAL(modelChanged()));
    emit modelChanged();
    dlg.exec();
  } else
    dlg.slotApply();
  return;
}

void KplDoc::openDocument(const char* filename, fileTypes type)
{
  int i;
  FILE *f;
  QString fn;

  KURL u = KURL(filename);
  if (u.isMalformed())
    u.parse(QDir::cleanDirPath(QDir::currentDirPath() + "/" + filename));
  if (u.isLocalFile())
    fn = u.path();
  else
    if (!KFM::download(u.url(), fn))
      return;
  fType = type;
  QFileInfo fi(u.path());
  if (fType == UNKNOWN) {
    QString ext = fi.extension();
    fType = stricmp(ext, "plo") ? DATAFILE : PLOTFILE;
  }
  if (fType == DATAFILE) {
    freeAllItd();
    nItems = aut.autoFit ? 3 : 2;
    itd = (PITEMDATA) calloc(nItems, sizeof(ITEMDATA));
    iniStru(&itd[0], FRAMEITEM);
    iniStru(&itd[1], ARRAYITEM);
    if (aut.autoFit)
      iniStru(&itd[2], FUNITEM);
    if (u.isLocalFile())
      strcpy(((PARRAYDATA) itd[1].stru)->path, fn);
    else
      strcpy(((PARRAYDATA) itd[1].stru)->path, u.url());
    if (!readFile(((PARRAYDATA) itd[1].stru)->path,
                  &((PARRAYDATA) itd[1].stru)->x,
                  &((PARRAYDATA) itd[1].stru)->ncols,
                  &((PARRAYDATA) itd[1].stru)->nrows)) {
      newDocument();
      KMsgBox::message((QWidget *) parent(), i18n("Error"),
                       i18n("while loading the file"), KMsgBox::EXCLAMATION);
      emit modelChanged();
      if (!u.isLocalFile())
        KFM::removeTempFile(fn);
      return;
    } else {
      setModified();
      autoStruc((PFRAMEDATA) itd[0].stru, (PARRAYDATA) itd[1].stru);
    }
  } else {
    if ((f = fopen(fn, "r"))) {
      newDocument();
      nItems = readPlo(f);
      if (nItems) {
        for (i = 0; i < nItems; i++) {
          switch (itd[i].ityp) {
            case ARRAYITEM:
              ((PARRAYDATA) itd[i].stru)->x = 0;
              if (!readFile(((PARRAYDATA) itd[i].stru)->path,
                            &((PARRAYDATA) itd[i].stru)->x,
                            &((PARRAYDATA) itd[i].stru)->ncols,
                            &((PARRAYDATA) itd[i].stru)->nrows))
                KMsgBox::message((QWidget *) parent(), i18n("Error"),
                                 (((PARRAYDATA) itd[i].stru)->path),
                                 KMsgBox::EXCLAMATION);
              break;
            case FUNITEM:
              getFun(((PFUNDATA) itd[i].stru)->path,
                     ((PFUNDATA) itd[i].stru)->name,
                     &((PFUNDATA) itd[i].stru)->hmod,
                     &((PFUNDATA) itd[i].stru)->fkt);
              break;
            case PARFUNITEM:
              getFun(((PPARFUNDATA) itd[i].stru)->pathx,
                     ((PPARFUNDATA) itd[i].stru)->namex,
                     &((PPARFUNDATA) itd[i].stru)->hmodx,
                     &((PPARFUNDATA) itd[i].stru)->fktx);
              getFun(((PPARFUNDATA) itd[i].stru)->pathy,
                     ((PPARFUNDATA) itd[i].stru)->namey,
                     &((PPARFUNDATA) itd[i].stru)->hmody,
                     &((PPARFUNDATA) itd[i].stru)->fkty);
              break;
            default:
              break;
          }
        }
      }
    } else {
      KMsgBox::message((QWidget *) parent(), i18n("Error"),
                       i18n("while loading the file"), KMsgBox::EXCLAMATION);
      return;
    }
  }
  if (u.isLocalFile()) {
    m_file = fi.absFilePath();
    m_dir = fi.dirPath(true);
  } else {
    m_file = u.url();
    m_dir = u.directoryURL();
    KFM::removeTempFile(fn);
  }
  if (fType == DATAFILE)
    if (aut.autoFit)
      autoFit((PARRAYDATA) itd[1].stru, (PFUNDATA) itd[2].stru);
  emit modelChanged();
}

bool KplDoc::getFun(QString path, QString name, void **hmod,
                    double (**fkt)(double, double *))
{
  char *error;
  QString msg;

  *hmod = 0;
  *fkt = 0;
  if (!path.isEmpty()) {
    if (!(*hmod = dlopen(path, RTLD_NOW))) {
      KMsgBox::message((QWidget *) parent(), i18n("Error"),
                       msg.sprintf(i18n("while trying to open\nfile %s:\n%s"),
                                   path.data(), dlerror()),
                       KMsgBox::EXCLAMATION);
      return true;
    } else
      if (!name.isEmpty())
        *fkt = (double (*)(double, double *)) dlsym(*hmod, name);
        if ((error = (char *) dlerror()) != 0) {
          KMsgBox::message((QWidget *) parent(), i18n("Error"),
                           msg.sprintf(i18n("while trying to determine the "
                                            "address of\nfunction %s:\n%s"),
                                       name.data(), error),
                           KMsgBox::EXCLAMATION);
          dlclose(*hmod);
          *hmod = 0;
          return true;
        }
  }
  return false;
}

int KplDoc::readPlo(FILE *f)
{
  const char framename[][9] = {"ndigx", "ndigy", "mticx", "mticy", "gridmode",
                               "iex", "iey", "colframe", "colgrid",
                               "xf", "yf", "xw0", "xw", "yw0", "yw",
                               "xmi", "xma", "ymi", "yma", "xtic", "ytic",
                               "relsiz", "sx", "sy", "sh"};
  const char arrayname[][8] = {"ix", "iy", "ie", "istart", "n", "symb",
                               "errbars", "color", "fx", "fy", "path"};
  const char funname[][6] = {"symb", "color", "xfmin", "xfmax", "dx", "fx",
                             "fy", "p", "name", "path"};
  const char parfunname[][6] = {"symb", "color", "tmin", "tmax", "dt", "fx",
                                "fy", "px", "py", "namex", "namey",
                                "pathx", "pathy"};
  const char legendname[][8] = {"symb", "colsymb", "coltext", "x", "y",
                                "xtext", "ytext", "relsiz", "s"};
  int i = -99, j, k;
  char line[256], *p;

  while (fgets(line, sizeof(line), f)) {
    if ((p = strchr(line, '\r')) != 0)
      *p = '\0';
    if ((p = strchr(line, '\n')) != 0)
      *p = '\0';
    if (line[0] == '\0')
      continue;
    if (line[0] == '[') {
      if ((p = strchr(line + 1, ']')) != 0) {
        *p = '\0';
        if (!strcmp(line + 1, "Items")) {
          if (i == -99)
            i = -1;
          else {
            nItems = 0;
            return nItems;
          }
        } else {
          if (!strncmp(line + 1, "Item ", 5))
            if (i == -99)
              return nItems;
            else {
              sscanf(line + 6, "%i", &i);
              if ((i < 0) || (i >= nItems)) {
                nItems = 0;
                return nItems;
              }
            }
        }
      }
    } else {
      if (i == -99)
        return nItems;
      strtok(line, "=");
      p = line + strlen(line) + 1;
      if (i == -1)
        if (strcmp(line, "nitems")) {
          nItems = 0;
          return nItems;
        } else {
          sscanf(p, "%i", &nItems);
          if (nItems <= 0) {
            nItems = 0;
            return nItems;
          } else
            itd = (PITEMDATA) calloc(nItems, sizeof(ITEMDATA));
        }
      else {
        j = strcmp(line, "Type");
        if (itd[i].ityp) {
          if (j) {
            switch (itd[i].ityp) {
              case FRAMEITEM:
                for (j = 0; j < 25; j++)
                  if (!strcmp(line, framename[j]))
                    break;
                if (j >= 25) {
                  nItems = 0;
                  return nItems;
                }
                if (j < 7) {
                  sscanf(p, "%i", &((PFRAMEDATA) itd[i].stru)->ndigx + j);
                  break;
                }
                if (j < 9) {
                  sscanf(p, "%lx", &((PFRAMEDATA)
                         itd[i].stru)->colFrame + j - 7);
                  break;
                }
                if (j < 22) {
                  sscanf(p, "%lg", &((PFRAMEDATA) itd[i].stru)->xf + j - 9);
                  break;
                }
                strcpy(((PFRAMEDATA) itd[i].stru)->sx + 80 * (j - 22), p);
                break;
              case ARRAYITEM:
                for (j = 0; j < 11; j++)
                  if (!strcmp(line, arrayname[j]))
                    break;
                if (j >= 25) {
                  nItems = 0;
                  return nItems;
                }
                if (j < 7) {
                  sscanf(p, "%i", &((PARRAYDATA) itd[i].stru)->ix + j);
                  break;
                }
                if (j < 8) {
                  sscanf(p, "%lx", &((PARRAYDATA) itd[i].stru)->color);
                  break;
                }
                if (j < 10) {
                  sscanf(p, "%lg", &((PARRAYDATA) itd[i].stru)->fx + j - 8);
                  break;
                }
                strcpy(((PARRAYDATA) itd[i].stru)->path, p);
                break;
              case FUNITEM:
                for (j = 0; j < 10; j++)
                  if (!strcmp(line, funname[j]))
                    break;
                if (j >= 10) {
                  nItems = 0;
                  return nItems;
                }
                if (j == 0) {
                  sscanf(p, "%i", &((PFUNDATA) itd[i].stru)->symb);
                  break;
                }
                if (j == 1) {
                  sscanf(p, "%lx", &((PFUNDATA) itd[i].stru)->color);
                  break;
                }
                if (j < 7) {
                  sscanf(p, "%lg", &((PFUNDATA) itd[i].stru)->xfmin + j - 2);
                  break;
                }
                if (j == 7) {
                  strtok(p, " ");
                  for (k = 0; k < NPMAX; k++) {
                    sscanf(p, "%lg", &((PFUNDATA) itd[i].stru)->p[k]);
                    p = strtok(0, " ");
                  }
                  break;
                }
                strcpy(((PFUNDATA) itd[i].stru)->name + 100 * (j - 8), p);
                break;
              case PARFUNITEM:
                for (j = 0; j < 13; j++)
                  if (!strcmp(line, parfunname[j]))
                    break;
                if (j >= 13) {
                  nItems = 0;
                  return nItems;
                }
                if (j == 0) {
                  sscanf(p, "%i", &((PPARFUNDATA) itd[i].stru)->symb);
                  break;
                }
                if (j == 1) {
                  sscanf(p, "%lx", &((PPARFUNDATA) itd[i].stru)->color);
                  break;
                }
                if (j < 7) {
                  sscanf(p, "%lg", &((PPARFUNDATA) itd[i].stru)->tmin + j - 2);
                  break;
                }
                if (j == 7) {
                  strtok(p, " ");
                  for (k = 0; k < NPMAX; k++) {
                    sscanf(p, "%lg", &((PPARFUNDATA) itd[i].stru)->px[k]);
                    p = strtok(0, " ");
                  }
                  break;
                }
                if (j == 8) {
                  strtok(p, " ");
                  for (k = 0; k < NPMAX; k++) {
                    sscanf(p, "%lg", &((PPARFUNDATA) itd[i].stru)->py[k]);
                    p = strtok(0, " ");
                  }
                  break;
                }
                if (j < 11) {
                  strcpy(((PPARFUNDATA) itd[i].stru)->namex + 100 * (j - 9),
                         p);
                  break;
                }
                strcpy(((PPARFUNDATA) itd[i].stru)->pathx + 260 * (j - 11),
                       p);
                break;
              case LEGENDITEM:
                for (j = 0; j < 9; j++)
                  if (!strcmp(line, legendname[j]))
                    break;
                if (j >= 9) {
                  nItems = 0;
                  return nItems;
                }
                if (j == 0) {
                  sscanf(p, "%i", &((PLEGENDDATA) itd[i].stru)->symb);
                  break;
                }
                if (j < 3) {
                  sscanf(p, "%lx", &((PLEGENDDATA)
                         itd[i].stru)->colsymb + j - 1);
                  break;
                }
                if (j < 8) {
                  sscanf(p, "%lg", &((PLEGENDDATA) itd[i].stru)->x + j - 3);
                  break;
                }
                strcpy(((PLEGENDDATA) itd[i].stru)->s, p);
                break;
            }
          } else {
            nItems = 0;
            return nItems;
          }
        } else {
          if (j) {
            nItems = 0;
            return nItems;
          } else {

            int l;
            for (l = FRAMEITEM; l <= LEGENDITEM; l++)
              if (!strcmp(p, itemName[l]))
                break;
            if (l > LEGENDITEM) {
              nItems = 0;
              return nItems;
            }
            itd[i].ityp = (itemTypes) l;
            itd[i].stru = calloc(iSize[l], 1);
          }
        }
      }
    }
  }
  fclose(f);
  return nItems;
}

void KplDoc::writePlo(FILE *f)
{
  int i, j;

  fprintf(f, "[Items]\nnitems=%i\n", nItems);
  for (i = 0; i < nItems; i++) {
    fprintf(f, "\n[Item %i]\nType=%s\n", i, itemName[itd[i].ityp]);
    switch (itd[i].ityp) {
      case FRAMEITEM:
        fprintf(f, "ndigx=%i\nndigy=%i\nmticx=%i\nmticy=%i\ngridmode=%i\n"
                "iex=%i\niey=%i\ncolframe=%lx\ncolgrid=%lx\n"
                "xf=%g\nyf=%g\nxw0=%g\nxw=%g\nyw0=%g\nyw=%g\n"
                "xmi=%g\nxma=%g\nymi=%g\nyma=%g\nxtic=%g\nytic=%g\n"
                "relsiz=%g\nsx=%s\nsy=%s\nsh=%s\n",
                ((PFRAMEDATA) itd[i].stru)->ndigx,
                ((PFRAMEDATA) itd[i].stru)->ndigy,
                ((PFRAMEDATA) itd[i].stru)->mticx,
                ((PFRAMEDATA) itd[i].stru)->mticy,
                ((PFRAMEDATA) itd[i].stru)->gridmode,
                ((PFRAMEDATA) itd[i].stru)->iex,
                ((PFRAMEDATA) itd[i].stru)->iey,
                ((PFRAMEDATA) itd[i].stru)->colFrame,
                ((PFRAMEDATA) itd[i].stru)->colGrid,
                ((PFRAMEDATA) itd[i].stru)->xf,
                ((PFRAMEDATA) itd[i].stru)->yf,
                ((PFRAMEDATA) itd[i].stru)->xw0,
                ((PFRAMEDATA) itd[i].stru)->xw,
                ((PFRAMEDATA) itd[i].stru)->yw0,
                ((PFRAMEDATA) itd[i].stru)->yw,
                ((PFRAMEDATA) itd[i].stru)->xmi,
                ((PFRAMEDATA) itd[i].stru)->xma,
                ((PFRAMEDATA) itd[i].stru)->ymi,
                ((PFRAMEDATA) itd[i].stru)->yma,
                ((PFRAMEDATA) itd[i].stru)->xtic,
                ((PFRAMEDATA) itd[i].stru)->ytic,
                ((PFRAMEDATA) itd[i].stru)->relsiz,
                ((PFRAMEDATA) itd[i].stru)->sx,
                ((PFRAMEDATA) itd[i].stru)->sy,
                ((PFRAMEDATA) itd[i].stru)->sh);
        break;
      case ARRAYITEM:
        fprintf(f, "ix=%i\niy=%i\nie=%i\nistart=%i\nn=%i\nsymb=%i\n"
                "errbars=%i\ncolor=%lx\nfx=%g\nfy=%g\npath=%s\n",
                ((PARRAYDATA) itd[i].stru)->ix,
                ((PARRAYDATA) itd[i].stru)->iy,
                ((PARRAYDATA) itd[i].stru)->ie,
                ((PARRAYDATA) itd[i].stru)->istart,
                ((PARRAYDATA) itd[i].stru)->n,
                ((PARRAYDATA) itd[i].stru)->symb,
                ((PARRAYDATA) itd[i].stru)->errbars,
                ((PARRAYDATA) itd[i].stru)->color,
                ((PARRAYDATA) itd[i].stru)->fx,
                ((PARRAYDATA) itd[i].stru)->fy,
                ((PARRAYDATA) itd[i].stru)->path);
         break;
      case FUNITEM:
        fprintf(f, "symb=%i\ncolor=%lx\nxfmin=%g\nxfmax=%g\ndx=%g\n"
                "fx=%g\nfy=%g\np=",
                ((PFUNDATA) itd[i].stru)->symb,
                ((PFUNDATA) itd[i].stru)->color,
                ((PFUNDATA) itd[i].stru)->xfmin,
                ((PFUNDATA) itd[i].stru)->xfmax,
                ((PFUNDATA) itd[i].stru)->dx,
                ((PFUNDATA) itd[i].stru)->fx,
                ((PFUNDATA) itd[i].stru)->fy);
        for (j = 0; j < 19; j++)
          fprintf(f, "%g ",((PFUNDATA) itd[i].stru)->p[j]);
        fprintf(f, "%g\nname=%s\npath=%s\n",
                ((PFUNDATA) itd[i].stru)->p[19],
                ((PFUNDATA) itd[i].stru)->name,
                ((PFUNDATA) itd[i].stru)->path);
        break;
      case PARFUNITEM:
        fprintf(f, "symb=%i\ncolor=%lx\ntmin=%g\ntmax=%g\ndt=%g\n"
                "fx=%g\nfy=%g\npx=",
                ((PPARFUNDATA) itd[i].stru)->symb,
                ((PPARFUNDATA) itd[i].stru)->color,
                ((PPARFUNDATA) itd[i].stru)->tmin,
                ((PPARFUNDATA) itd[i].stru)->tmax,
                ((PPARFUNDATA) itd[i].stru)->dt,
                ((PPARFUNDATA) itd[i].stru)->fx,
                ((PPARFUNDATA) itd[i].stru)->fy);
        for (j = 0; j < 19; j++)
          fprintf(f, "%g ",((PPARFUNDATA) itd[i].stru)->px[j]);
        fprintf(f, "%g\npy=",((PPARFUNDATA) itd[i].stru)->px[19]);
        for (j = 0; j < 19; j++)
          fprintf(f, "%g ",((PPARFUNDATA) itd[i].stru)->py[j]);
        fprintf(f, "%g\nnamex=%s\nnamey=%s\npathx=%s\npathy=%s\n",
                ((PPARFUNDATA) itd[i].stru)->py[19],
                ((PPARFUNDATA) itd[i].stru)->namex,
                ((PPARFUNDATA) itd[i].stru)->namey,
                ((PPARFUNDATA) itd[i].stru)->pathx,
                ((PPARFUNDATA) itd[i].stru)->pathy);
        break;
      case LEGENDITEM:
        fprintf(f, "symb=%i\ncolsymb=%lx\ncoltext=%lx\nx=%g\ny=%g\n"
                "xtext=%g\nytext=%g\nrelsiz=%g\ns=%s\n",
                ((PLEGENDDATA) itd[i].stru)->symb,
                ((PLEGENDDATA) itd[i].stru)->colsymb,
                ((PLEGENDDATA) itd[i].stru)->coltext,
                ((PLEGENDDATA) itd[i].stru)->x,
                ((PLEGENDDATA) itd[i].stru)->y,
                ((PLEGENDDATA) itd[i].stru)->xtext,
                ((PLEGENDDATA) itd[i].stru)->ytext,
                ((PLEGENDDATA) itd[i].stru)->relsiz,
                ((PLEGENDDATA) itd[i].stru)->s);
    }
  }
  fclose(f);
}

int KplDoc::minMaxFl(double *xmin, double *xmax, double *x, int n)
{
  int i;

  *xmin = 1.0e300;
  *xmax = -*xmin;
  for (i = 0; i < n; i++) {
    *xmin = min(*xmin, x[i]);
    *xmax = max(*xmax, x[i]);
  }
  return (*xmax == *xmin);
}

void KplDoc::expo(double a, int *ie, double *f)
{
  if((*ie = int9(log10(1.01 * a + 1.0e-37))) < 0)
    *ie = *ie - 1;
  *ie = -3 * (*ie / 3);
  *f = pow(10.0, *ie);
}

void KplDoc::expoFl(int *ie, double *f, double *x, int n)
{
  double xmin, xmax;

  minMaxFl(&xmin, &xmax, x, n);
  expo(max(fabs(xmin), fabs(xmax)), ie, f);
}

void KplDoc::expoFn(double (*fkt)(double, double *), int *ie, double *f,
                    double xmin, double xmax, double dx, double *p)
{
  double x, ymax;

  ymax = 0.0;
  if (!dx)
    dx = 0.01 * (xmax - xmin);
  x = xmin - dx;
  do {
    x += dx;
    if (fabs(x - xmax) < 0.9 * fabs(dx))
      x = xmax;
    ymax = max(ymax, fabs(fkt(x, p)));
  } while (x != xmax);
  expo(ymax, ie, f);
}

bool KplDoc::autoSc(double *xmi, double *xma, double *tic, int *mtic, int *ndig,
                    double xmin, double xmax, double fx, double c)
{
  double d, f;

  if (xmax == xmin)
    return true;
  d = c * fx * (xmax - xmin);
  f = pow(10.0, (- (float) (int9(log10(d + 1.0e-37)))));
  d = f * d;
  *mtic = 1;
  if(d > 1.414)
    *mtic = 2;
  if(d > 3.162)
    *mtic = 5;
  if(d > 7.071)
    *mtic = 10;
  *tic = *mtic / f;
  if((*mtic == 1) || (*mtic == 10))
    *mtic = 2;
  if(!(*ndig = -int9(log10(*tic + 1.0e-37) + 1.0e-6)))
    *ndig = -1;
  *xmi = *tic * int9(fx * xmin / *tic + 1.0e-4);
  *xma = *tic * int9(fx * xmax / *tic + 0.9999);
  return false;
}

bool KplDoc::autoFl(double *xmi, double *xma, double *tic, int *mtic, int *ndig,
                    int *ie, double *fn, double *x, int n, int norm, double c)
{
  double xmin, xmax;

  minMaxFl(&xmin, &xmax, x, n);
  if (norm)
    expo(max(fabs(xmin), fabs(xmax)), ie, fn);
  else {
    *ie = 0;
    *fn = 1.0;
  }
  if (c)
    return autoSc(xmi, xma, tic, mtic, ndig, xmin, xmax, *fn, c);
  else {
    *xmi = xmin;
    *xma = xmax;
    return (xmin == xmax);
  }
}

bool KplDoc::autoFn(double *ymi, double *yma, double *tic, int * mtic, int *ndig,
                    double (*fkt)(double, double *), double xmin, double xmax,
                    double dx, double *p, double fn, double c)
{
  double x, y, ymin, ymax;

  ymin = 1.0e300;
  ymax = -ymin;
  if(!dx)
    dx = 0.01 * (xmax - xmin);
  x = xmin - dx;
  do {
    x += dx;
    if (fabs(x - xmax) < 0.9 * fabs(dx))
      x = xmax;
    y = fn * fkt(x, p);
    ymin = min(ymin, y);
    ymax = max(ymax, y);
  } while (x != xmax);
  if (c)
    return autoSc(ymi, yma, tic, mtic, ndig, ymin, ymax, 1.0, c);
  else {
    *ymi = ymin;
    *yma = ymax;
    return (ymin == ymax);
  }
}

int KplDoc::int9(double a)
{
  int i = (int) a;
  if(a < 0.0)
    i--;
  return i;
}

void KplDoc::connectKFM()
{
  connect(kfm, SIGNAL(finished()), SLOT(slotKFM()));
}

void KplDoc::slotKFM()
{
  delete kfm;
  remove(QDir::homeDirPath() + "/kpltmp.tmp");
}
