/***************************************************************************
                          kpldoc.h  -  description                              
                             -------------------                                         

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sat Apr 24 15:14:00 MEST 1999
                                           
    copyright            : (C) 1999 by Werner Stille                         
    email                : stille@uni-freiburg.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#ifndef KPLDOC_H
#define KPLDOC_H 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

#include <stdio.h>
#include <qobject.h>
#include "kplots.h"

class KFM;

/**
  * Kpl model class.
  * Provides methods for reading, writing and scaling of plot data.
  * @author Werner Stille
  */
class KplDoc : public QObject
{
  Q_OBJECT

public:
  /**
   * Constructor.
   * @param parent pointer to parent QObject.
   */
  KplDoc(QObject *parent);
  /** Destructor. */
  ~KplDoc();
	/** Initializes the document generally. */
	void newDocument();
  /**
   * Saves the document under filename.
   * @param filename file path.
   */
  bool saveDocument(const char* filename);
  /**
   * Sets the modified flag for the document after a modifying action on the
   * view connected to the document.
   * @param modified true for document modified after last save action.
   */
  void setModified(bool modified = true);
  /**
   * Returns if the document is modified or not. Use this to determine if your
   * document needs saving by the user on closing.
   * @return true for document modified after last save action.
   */
  bool isModified();
  /**
   * Sets the current directory path.
   * @param dir current directory path.
   */
  void setCurrentDir(const char *dir);
  /**
   * Returns the current path.
   * @return current directory path.
   */
  const QString& currentDir() const;
	/**
	 * Sets the absolute filename of the document.
   * @param file absolute file path.
	 */
	void setCurrentFile(const char* file);
	/**
	 * Returns the absolute filename of the document.
   * @return absolute file path.
	 */
	const QString& currentFile() const;
  /**
   * Deallocates storage of a plot item.
   * @param iti pointer to item structure.
   */
  void freeItd(PITEMDATA iti);
  /** Deallocates storage of all structures. */
  void freeAllItd();
  /**
   * Deallocates array storage.
   * @param x pointer to array
   */
  void freeX(double ***x);
  /**
   * Deallocates array storage of a structure, when this is the only one using the array.
   * @param ad pointer to array data structure.
   */
  void freeXCond(PARRAYDATA ad);
  /**
   * Reads double values from a file into an array.
   * @param filename file path.
   * @param x pointer to two-dimensional double array.
   * @param ncols pointer to number of colums (output).
   * @param nrows pointer to number of lines (output).
   * @return number of lines, 0 for error.
   */
  int readFile(QString filename, double ***x, int *ncols, int *nrows);
  /**
   * Initializes structure.
   * @param itd pointer to structure.
   * @param ityp type of structure. One of the following:
   * FRAMEITEM, ARRAYITEM, FUNITEM, PARFUNITEM, LEGENDITEM.
   */
  void iniStru(PITEMDATA itd, itemTypes ityp);
  /**
   * Sets array and data structure elements by autoscaling.
   * @param fd pointer to frame data structure.
   * @param ad pointer to array data structure.
   * @return true for error
   */
  bool autoStruc(PFRAMEDATA fd, PARRAYDATA ad);
  /**
   * Sets function structure elements for autofit.
   * @param ad pointer to array data structure.
   * @param fd pointer to function data structure.
   */
  void autoFit(PARRAYDATA ad, PFUNDATA fd);
  /**
   * Reads file and initializes structures.
   * @param filename filename or URL.
   * @param type one of the following: UNKNOWN, DATAFILE, PLOTFILE.
   */
  void openDocument(const char* filename, fileTypes type = UNKNOWN);
  /**
   * Loads shared library and determines function addresses.
   * @param path file path.
   * @param name function name.
   * @param hmod pointer to module handle (output).
   * @param fkt pointer to function (output).
   * @return true for error.
   */
  bool getFun(QString path, QString name, void **hmod,
              double (**fkt)(double, double *));
  /**
   * Reads plot file and initializes structures.
   * @param f pointer to stream opened for write access
   * @return number of plot items
   */
  int readPlo(FILE *f);
  /**
   * Writes plot file.
   * @param f pointer to stream opened for write access.
   */
  void writePlo(FILE *f);
  /**
   * Determines minimum and maximum values of a data array.
   * @param xmin minimum value (output).
   * @param xmax maximum value (output).
   * @param x double array containg data in user units.
   * @param n number of data values.
   * @return true for xmax = xmin.
   */
  int minMaxFl(double *xmin, double *xmax, double *x, int n);
  /**
   * Calculates normalization factor and exponents of 10.
   * @param a maximum absolute value of data.
   * @param ie logarithm of normalization factor.
   * @param f normalization factor for data values.
   */
  void expo(double a, int *ie, double *f);
  /**
   * Calculates normalization factor for data arrays.
   * @param ie logarithm of normalization factor (output).
   * @param f normalization factor (output).
   * @param x double array containg data in user units.
   * @param n number of data values.
   */
  void expoFl(int *ie, double *f, double *x, int n);
  /**
   * Calculates normalization factor for functions.
   * @param fkt double function of double argument.
   * @param ie logarithm of normalization factor (output).
   * @param f normalization factor (output).
   * @param xmin start value of the argument.
   * @param xmax final value of the argument.
   * @param dx step size of argument (0.0 for automatic determination).
   * @param p pointer to array of parameters.
   */
  void expoFn(double (*fkt)(double, double *), int *ie, double *f,
              double xmin, double xmax, double dx, double *p);
  /**
   * Calculates scaling parameters.
   * @param xmi rounded minimum (output).
   * @param xma rounded maximum (output).
   * @param tic distance between major tics (output).
   * @param mtic number of intervals between major tics (output).
   * @param ndig number of places after decimal point for lettering (output).
   * @param xmin minimum value.
   * @param xmax maximum value.
   * @param fx normalization factor for data values.
   * @param c scaling constant, desired value for tic / (xma - xmi).
   * @return true for xmax = xmin.
   */
  bool autoSc(double *xmi, double *xma, double *tic, int *mtic, int *ndig,
             double xmin, double xmax, double fx, double c);
  /**
   * Calculates scaling parameters for functions.
   * @param ymi rounded minimum (output).
   * @param yma rounded maximum (output).
   * @param tic distance between major tics (output).
   * @param mtic number of intervals between major tics (output).
   * @param ndig number of places after decimal point for lettering (output).
   * @param fkt double function of double argument.
   * @param xmin start value of the argument.
   * @param xmax final value of the argument.
   * @param dx step size of argument (0.0 for automatic determination).
   * @param p pointer to array of parameters.
   * @param fn normalization factor for function values (input).
   * @param c scaling constant, desired value for tic / (xma - xmi).
   * Only xmi and xma are calculated when c = 0.
   * @return true for constant function values.
   */
  bool autoFn(double *ymi, double *yma, double *tic, int * mtic, int *ndig,
             double (*fkt)(double, double *), double xmin, double xmax,
             double dx, double *p, double fn, double c);
  /**
   * Calculates scaling parameters for data arrays.
   * @param xmi rounded minimum (output).
   * @param xma rounded maximum (output).
   * @param tic distance between major tics (output).
   * @param mtic number of intervals between major tics (output).
   * @param ndig number of places after decimal point for lettering (output).
   * @param ie logarithm of normalization factor (output).
   * @param fn normalization factor for data values (output).
   * @param x double array containg data in user units.
   * @param n number of data values.
   * @param norm do normalization if true.
   * @param c scaling constant, desired value for tic / (xma - xmi).
   * Only xmi and xma are calculated when c = 0.
   * @return true for constant data values.
   */
  bool autoFl(double *xmi, double *xma, double *tic, int *mtic, int *ndig,
             int *ie, double *fn, double *x, int n, int norm, double c);
  /** Connects KFM signal finished() to destroy kfm object and uploaded temporary file. */
  void connectKFM();
  static const int iSize[6];
  int nItems;
  fileTypes fType;
  PITEMDATA itd;
  BasicStruct bas;
  AutoStruct aut;
  KFM *kfm;

public slots:
  /** Deletes KFM object and temporary file for upload. */
  void slotKFM();

signals:
  /** Emitted when changes may require updating the menus and repainting the page view. */
  void modelChanged();

private:
 	/** The modified flag of the current document. */
  bool m_modified;
  QString m_file;
  QString m_dir;
  int int9(double a);
  static const char itemName[6][11];
  int nyLeg;
};

#endif // KPLDOC_H
