/***************************************************************************
                          kplview.cpp  -  description                              
                             -------------------                                         

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sat Apr 24 15:14:00 MEST 1999
                                           
    copyright            : (C) 1999 by Werner Stille                         
    email                : stille@uni-freiburg.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#include <qscrollbar.h>
#include <qpaintdevicemetrics.h>
#include <qpixmap.h>
#include "kplview.h"
#include "kpldoc.h"
#include "kpl.h"
#include "kplwidget.h"
#include "pslib.h"

#ifndef min
  #define min(x,y) ((x) < (y) ? (x) : (y))
#endif
#ifndef max
  #define max(x,y) ((x) > (y) ? (x) : (y))
#endif

KplView::KplView(KplApp *parent, double f) :
 QWidget(parent)
{
  doc = parent->getDocument();
  fZoom = f;
  fxn = fyn = 1.0;
  page = new KplWidget(this);
  sHScrollPos = sVScrollPos = 0;
  horScrollBar = new QScrollBar(QScrollBar::Horizontal, this);
  vertScrollBar = new QScrollBar(QScrollBar::Vertical, this);
  connect(horScrollBar, SIGNAL(valueChanged(int)),
          SLOT(slotHorScroll(int)));
  connect(vertScrollBar, SIGNAL(valueChanged(int)),
          SLOT(slotVertScroll(int)));
}

void KplView::print(QPaintDevice *printer)
{
  plInit(printer);
  doDraw();
  end();
}

void KplView::layout()
{
  bool hScroll = false, vScroll = false;

  QPaintDeviceMetrics pdm(this);
  xPict = int(10.0 * fZoom * doc->bas.xf * pdm.width() / pdm.widthMM() + 0.5);
  yPict = int(10.0 * fZoom * doc->bas.yf * pdm.height() / pdm.heightMM() + 0.5);
  scrx = width();
  scry = height();
  if (xPict > scrx) {
    hScroll = true;
    scry -= SCROLLBAR_SIZE;
  }
  if (yPict > scry) {
    vScroll = true;
    scrx -= SCROLLBAR_SIZE;
    if (!hScroll)
      if (xPict > scrx) {
        hScroll = true;
        scry -= SCROLLBAR_SIZE;
      }
  }
  if (hScroll) {
    horScrollBar->setGeometry(0, scry, scrx, SCROLLBAR_SIZE);
    horScrollBar->setRange(0, xPict - scrx);
    horScrollBar->setSteps(10, scrx);
    horScrollBar->setValue(sHScrollPos);
    horScrollBar->show();
  } else {
    horScrollBar->hide();
    sHScrollPos = 0;
  }
  if (vScroll) {
    vertScrollBar->setGeometry(scrx, 0, SCROLLBAR_SIZE, scry);
    vertScrollBar->setRange(0, yPict - scry);
    vertScrollBar->setSteps(10, scry);
    vertScrollBar->setValue(yPict - scry - sVScrollPos);
    vertScrollBar->show();
  } else {
    vertScrollBar->hide();
    sVScrollPos = 0;
  }
  page->resize(scrx, scry);
}

void KplView::doDraw()
{
  int i;

  for (i = 0; i < doc->nItems; i++)
    switch (doc->itd[i].ityp) {
      case FRAMEITEM:
        drawFrame((PFRAMEDATA) doc->itd[i].stru);
        break;
      case ARRAYITEM:
        drawArray((PARRAYDATA) doc->itd[i].stru);
        break;
      case FUNITEM:
        drawFun((PFUNDATA) doc->itd[i].stru);
        break;
      case PARFUNITEM:
        drawParFun((PPARFUNDATA) doc->itd[i].stru);
        break;
      case LEGENDITEM:
        drawLegend((PLEGENDDATA) doc->itd[i].stru);
    }
}

void KplView::doPS(FILE *f)
{
  int i;

  if (doc->nItems) {
    psInit(f, doc->bas.xf, doc->bas.yf);
    for (i = 0; i < doc->nItems; i++)
      switch (doc->itd[i].ityp) {
        case FRAMEITEM:
          psFrame((PFRAMEDATA) doc->itd[i].stru);
          break;
        case ARRAYITEM:
          psArray((PARRAYDATA) doc->itd[i].stru);
          break;
        case FUNITEM:
          psFun((PFUNDATA) doc->itd[i].stru);
          break;
        case PARFUNITEM:
          psParFun((PPARFUNDATA) doc->itd[i].stru);
          break;
        case LEGENDITEM:
          psLegend((PLEGENDDATA) doc->itd[i].stru);
      }
    psExit();
  }
}

void KplView::eraseWidget()
{
  doPaint(false);
}

void KplView::doPaint(bool paint)
{
  page->setCursor(waitCursor);
  page->pm->resize(page->size());
  plInit(page->pm, sHScrollPos, sVScrollPos);
  if (paint) {
    factor = fZoom;
    format(doc->bas.xf, doc->bas.yf);
    doDraw();
  }
  end();
  bitBlt(page, 0, 0, page->pm);
  page->setCursor(crossCursor);
}

void KplView::drawFrame(PFRAMEDATA fd)
{
  if (fd->xw && fd->yw) {
    format(fd->xf, fd->yf);
    relsize(fd->relsiz);
    Window(fd->xw0, fd->xw, fd->yw0, fd->yw);
    scale(fd->xmi, fd->xma, fd->ymi, fd->yma);
    setdig(fd->ndigx, fd->ndigy);
    setColFrame(QColor(fd->colFrame));
    setColGrid(QColor(fd->colGrid));
    raster(fd->xtic, fd->ytic, fd->mticx, fd->mticy, fd->gridmode);
    letx(fd->sx, fd->iex);
    lety(fd->sy, fd->iey);
    leth(fd->sh);
  }
}

void KplView::drawArray(PARRAYDATA ad)
{
  if (ad->x) {
    setColData(QColor(ad->color));
    symbol(ad->symb);
    plarry(&ad->x[ad->ix][ad->istart], &ad->x[ad->iy][ad->istart],
           ad->fx, ad->fy, ad->n);
    if (ad->errbars)
      plerry(&ad->x[ad->ix][ad->istart], &ad->x[ad->iy][ad->istart],
             &ad->x[ad->ie][ad->istart], ad->fx, ad->fy, ad->n);
    fxn = ad->fx;
    fyn = ad->fy;
  }
}

void KplView::drawFun(PFUNDATA fd)
{
  if (fd->fkt) {
    setColData(QColor(fd->color));
    symbol(fd->symb);
    plfun(fd->xfmin, fd->xfmax, fd->dx, fd->fx, fd->fy, fd->fkt, fd->p);
  }
}

void KplView::drawParFun(PPARFUNDATA pfd)
{
  if (pfd->fktx && pfd->fkty) {
    setColData(QColor(pfd->color));
    symbol(pfd->symb);
    plpfun(pfd->tmin, pfd->tmax, pfd->dt, pfd->fx, pfd->fy,
           pfd->fktx, pfd->px, pfd->fkty, pfd->py);
  }
}

void KplView::drawLegend(PLEGENDDATA ld)
{
  int ix, iy, n;
  double x[2], y[2], sav;

  setColData(QColor(ld->colsymb));
  symbol(ld->symb);
  if (ld->symb > 0) {
    cm2r(ld->x - 0.4 * ld->xtext, ld->y, 1.0, 1.0, &x[0], &y[0]);
    cm2r(ld->x + 0.4 * ld->xtext, ld->y, 1.0, 1.0, &x[1], &y[1]);
    n = 2;
  } else {
    cm2r(ld->x, ld->y, 1.0, 1.0, &x[0], &y[0]);
    n = 1;
  }
  plarry(x, y, 1.0, 1.0, n);
  setColFrame(QColor(ld->coltext));
  cm2r(ld->x + ld->xtext, ld->y + ld->ytext + 0.18, 1.0, 1.0, &x[0], &y[0]);
  r2i(x[0], y[0], &ix, &iy);
  sav = relsize(ld->relsiz);
  text(ld->s, ix, iy, 1);
  relsize(sav);
}

void KplView::psFrame(PFRAMEDATA fd)
{
  if (fd->xw && fd->yw) {
    psFormat(fd->xf, fd->yf);
    psRelSize(fd->relsiz);
    psWindow(fd->xw0, fd->xw, fd->yw0, fd->yw);
    psScale(fd->xmi, fd->xma, fd->ymi, fd->yma);
    psSetDig(fd->ndigx, fd->ndigy);
    psSetColFrame(fd->colFrame);
    psSetColGrid(fd->colGrid);
    psRaster(fd->xtic, fd->ytic, fd->mticx, fd->mticy, fd->gridmode);
    psLetX(fd->sx, fd->iex);
    psLetY(fd->sy, fd->iey);
    psLetH(fd->sh);
  }
}

void KplView::psArray(PARRAYDATA ad)
{
  if (!ad->x)
    return;
  psSetColData(ad->color);
  psSymbol(ad->symb);
  psPlArray(&ad->x[ad->ix][ad->istart], &ad->x[ad->iy][ad->istart],
            ad->fx, ad->fy, ad->n);
  if (ad->errbars)
    psPlError(&ad->x[ad->ix][ad->istart], &ad->x[ad->iy][ad->istart],
              &ad->x[ad->ie][ad->istart], ad->fx, ad->fy, ad->n);
}

void KplView::psFun(PFUNDATA fd)
{
  if (fd->fkt) {
    psSetColData(fd->color);
    psSymbol(fd->symb);
    psPlFun(fd->xfmin, fd->xfmax, fd->dx, fd->fx, fd->fy, fd->fkt, fd->p);
  }
}

void KplView::psParFun(PPARFUNDATA pfd)
{
  if (pfd->fktx && pfd->fkty) {
    psSetColData(pfd->color);
    psSymbol(pfd->symb);
    psPlParFun(pfd->tmin, pfd->tmax, pfd->dt, pfd->fx, pfd->fy,
             pfd->fktx, pfd->px, pfd->fkty, pfd->py);
  }
}

void KplView::psLegend(PLEGENDDATA ld)
{
  const double f = 28.34645669;
  int n;
  double x[2], y[2], sav;

  psSetColData(ld->colsymb);
  psSymbol(ld->symb);
  if (ld->symb > 0) {
    cm2r(ld->x - 0.4 * ld->xtext, ld->y, 1.0, 1.0, &x[0], &y[0]);
    cm2r(ld->x + 0.4 * ld->xtext, ld->y, 1.0, 1.0, &x[1], &y[1]);
    n = 2;
  } else {
    cm2r(ld->x, ld->y, 1.0, 1.0, &x[0], &y[0]);
    n = 1;
  }
  psPlArray(x, y, 1.0, 1.0, n);
  psSetColFrame(ld->coltext);
  sav = psRelSize(ld->relsiz);
  psText(ld->s, f * (ld->x + ld->xtext), f * (ld->y + ld->ytext), 1);
  psRelSize(sav);
}

void KplView::slotScrollLeft()
{
  horScrollBar->subtractLine();
}

void KplView::slotScrollRight()
{
  horScrollBar->addLine();
}

void KplView::slotScrollUp()
{
  vertScrollBar->subtractLine();
}

void KplView::slotScrollDown()
{
  vertScrollBar->addLine();
}

void KplView::slotScrollPageUp()
{
  vertScrollBar->subtractPage();
}

void KplView::slotScrollPageDown()
{
  vertScrollBar->addPage();
}

void KplView::slotScrollPageLeft()
{
  horScrollBar->subtractPage();
}

void KplView::slotScrollPageRight()
{
  horScrollBar->addPage();
}

void KplView::slotScrollHomeLeft()
{
  horScrollBar->setValue(0);
}

void KplView::slotScrollHomeDown()
{
  vertScrollBar->setValue(vertScrollBar->maxValue());
}

void KplView::slotScrollEndRight()
{
  horScrollBar->setValue(horScrollBar->maxValue());
}

void KplView::slotScrollEndUp()
{
  vertScrollBar->setValue(0);
}

void KplView::paintWidget()
{
  doPaint(true);
}

void KplView::resizeEvent(QResizeEvent *)
{
  layout();
  paintWidget();
}

void KplView::slotHorScroll(int pos)
{
  sHScrollPos = pos;
  paintWidget();
}

void KplView::slotVertScroll(int pos)
{
  sVScrollPos = vertScrollBar->maxValue() - pos;
  paintWidget();
}


