/***************************************************************************
                          kplview.h  -  description                              
                             -------------------                                         

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sat Apr 24 15:14:00 MEST 1999
                                           
    copyright            : (C) 1999 by Werner Stille                         
    email                : stille@uni-freiburg.de                                     
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   * 
 *                                                                         *
 ***************************************************************************/

#ifndef KPLVIEW_H
#define KPLVIEW_H 

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif 

#include <stdio.h>
#include <qwidget.h>
#include "kgraph.h"
#include "kplots.h"

#define SCROLLBAR_SIZE 16

class QScrollBar;
class KApplication;
class KplApp;
class KplWidget;
class KplDoc;

/**
  * Kpl main view class.
  * Provides methods for painting and erasing the page widget and generating PostScript files.
  * @author Werner Stille
  */
class KplView : public QWidget, public KGraph
{
  Q_OBJECT

public:
  /**
   * Constructor.
   * @param parent pointer to parent widget.
   * @param f zoom factor.
   */
  KplView(KplApp *parent, double f);
  /**
   * Contains the implementation for printing functionality.
   * @param printer printer device.
   */
	void print(QPaintDevice *printer);
  /** Adjusts scrollbars and page widget size. */
  void layout();
  /** Plots frame, grid, labels, data arrays, ordinary and parametric functions. */
  void doDraw();
  /**
   * Generates PostScript file containing frame grid, labels, arrays, ordinary and
   * parametric functions.
   * @param f pointer to stream opened for write access.
   */
  void doPS(FILE *f);
  /** Erases page widget. */
  void eraseWidget();
  /**
   * Paints or erases page widget.
   * @param paint true for painting.
   */
  void doPaint(bool paint);
  /**
   * Plots frame.
   * @param fd pointer to frame data structure.
   */
  void drawFrame(PFRAMEDATA fd);
  /**
   * Plots data array.
   * @param ad pointer to array data structure.
   */
  void drawArray(PARRAYDATA ad);
  /**
   * Plots ordinary function.
   * @param fd pointer to function data structure.
   */
  void drawFun(PFUNDATA fd);
  /**
   * Plots parametric function.
   * @param pfd pointer to parametric function data structure.
   */
  void drawParFun(PPARFUNDATA pfd);
  /**
   * Plots legend.
   * @param ld pointer to function legend structure.
   */
  void drawLegend(PLEGENDDATA ld);
  /**
   * Generates PostScript code for frame.
   * @param fd pointer to frame data structure.
   */
  void psFrame(PFRAMEDATA fd);
  /**
   * Generates PostScript code for array.
   * @param ad pointer to array data structure.
   */
  void psArray(PARRAYDATA ad);
  /**
   * Generates PostScript code for ordinary function.
   * @param fd pointer to function data structure.
   */
  void psFun(PFUNDATA fd);
  /**
   * Generates PostScript code for parametric function.
   * @param pfd pointer to parametric function data structure.
   */
  void psParFun(PPARFUNDATA pfd);
  /**
   * Generates PostScript code for legend.
   * @param ld pointer to legend data structure.
   */
  void psLegend(PLEGENDDATA ld);
  KplWidget *page;
  QScrollBar *horScrollBar, *vertScrollBar;
  int sHScrollPos, sVScrollPos;
  int xPict, yPict, scrx, scry;
  double fxn, fyn;
  double fZoom;

public slots:
  /** Scrolls left. */
  void slotScrollLeft();
  /** Scrolls right. */
  void slotScrollRight();
  /** Scrolls up. */
  void slotScrollUp();
  /** Scrolls down. */
  void slotScrollDown();
  /** Scrolls page up. */
  void slotScrollPageUp();
  /** Scrolls page down. */
  void slotScrollPageDown();
  /** Scrolls page left. */
  void slotScrollPageLeft();
  /** Scrolls page right. */
  void slotScrollPageRight();
  /** Scrolls to left margin. */
  void slotScrollHomeLeft();
  /** Scrolls to bottom margin. */
  void slotScrollHomeDown();
  /** Scrolls to right margin. */
  void slotScrollEndRight();
  /** Scrolls to top margin. */
  void slotScrollEndUp();
  /** Paints the page widget. */
  void paintWidget();

protected:
  /**
   * Overloaded for adjusting of scrollbars and page widget and
   * repainting of page widget.
   */
  virtual void resizeEvent(QResizeEvent *);

protected slots:
  /**
   * Updates page widget according to horizontal scrolling.
   * @param pos new horizontal scrollbar position.
   */
  void slotHorScroll(int pos);
  /**
   * Updates page widget according to vertical scrolling.
   * @param pos new vertical scrollbar position.
   */
  void slotVertScroll(int pos);

private:
  KplDoc *doc;
};

#endif // KPLVIEW_H 
