/***************************************************************************
                          plslib.h  -  description
                             -------------------

    This file is a part of kpl - a program for graphical presentation of
    data sets and functions.

    begin                : Sun Apr 25 1999
    copyright            : (C) 1999 by Werner Stille
    email                : stille@uni-freiburg.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PSLIB_H
#define PSLIB_H

#if defined(_Windows) || defined(__WIN32__)
  #include <windows.h>
#endif

#ifdef __GNUC__
  #ifdef __WIN32__
    #define _api WINAPI
  #else
    #define _api
    #define FAR
  #endif
#else
  #define _api FAR WINAPI
#endif

#ifdef __cplusplus
extern "C" {
#endif

/**
 * Initializes PostScript graphics.
 * @param f pointer to stream opened for write access.
 * @param xf width of bounding box.
 * @param yf height of bounding box.
 */
extern void psInit(FILE *f, double xf, double yf);
/**
 * Sets page format.
 * @param x page width in cm.
 * @param y page height in cm.
 */
extern void psFormat(double x, double y);
/**
 * Sets relative size of characters and symbols.
 * @param r new value of relative size.
 * @returns old value of relative size
 */
extern double psRelSize(double r);
/**
 * Sets size and position of data window.
 * @param x0 distance between left margin and page margin.
 * @param xl width.
 * @param y0 distance between lower margin and page margin.
 * @param yl height.
 */
extern void psWindow(double x0, double xl, double y0, double yl);
/**
 * Sets user units.
 * @param xl x value of left margin of data window.
 * @param xr x value of right margin of data window.
 * @param yb y value of lower margin of data window.
 * @param yt y value of upper margin of data window.
 */
extern void psScale(double xl, double xr, double yb, double yt);
/**
 * Sets number of digits after decimal point for axis labels.
 * @param nx number of digits for x values
 * (-1: no output of decimal point).
 * @param ny number of digits for y values
 * (-1: no output of decimal point.
 */
extern void psSetDig(int nx, int ny);
/**
 * Sets color for frame and labels.
 * @param icol color
 */
extern void psSetColFrame(long icol);
/**
 * Sets color for grid.
 * @param icol color
 */
extern void psSetColGrid(long icol);
/**
 * Sets color for data plot.
 * @param icol color
 */
extern void psSetColData(long icol);
/** Sets all colors.
 * @param icol1 color for frame and labels.
 * @param icol2 color for grid.
 * @param icol3 color for data plot.
 */
extern void psSetCol(long icol1, long icol2, long icol3);
/** Draws frame around data window. */
extern void psFrame(void);
/**
 * Draws frame and axes of data window.
 * @param xtic distance between major tics of x axis.
 * @param ytic distance between major tics of y axis.
 * @param intx number of intervals between major tics of x axis.
 * @param inty number of intervals between major tics of y axis.
 * @param mode 0 for frame only, 1 for axes with tics,
 * 2 for axes with tics and labels, 3 for grid, 4 for grid with labels.
 */
extern void psRaster(double xtic, double ytic, int intx, int inty, int mode);
/**
 * Plots text.
 * @param str text.
 * @param ix x value in integer units.
 * @param iy y value in integer units.
 * @param iorg 1: left aligned, 2: centered, 3: right aligned.
 */
extern void psText(char *str, double x, double y, int iorg);
/**
 * Labels head of the plot.
 * @param str text
 */
extern void psLetH(char *str);
/**
 * Labels x axis.
 * @param str text.
 * @param iex logarithm of normalization factor.
 */
extern void psLetX(char *str, int iex);
/**
 * Labels y axis.
 * @param str text.
 * @param iey logarithm of normalization factor.
 */
extern void psLetY(char *str, int iey);
/**
 * Selects marker or line type.
 * @param is 0 for circles, < 0 for different markers,
 * >0 for different line types.
 */
extern void psSymbol(int is);
/**
 * Draws markers or line given by an array of data points.
 * @param x array of x values in user units.
 * @param y array of y values in user units.
 * @param fx normalization factor for x values.
 * @param fy normalization factor for y values.
 * @param n number of points.
 */
extern void psPlArray(double *x, double *y, double fx, double fy, int n);
/**
 * Plots error bars for an array of data points.
 * @param x array of x values in user units.
 * @param y array of y values in user units.
 * @param e array of errors of the y values in user units.
 * @param fx normalization factor for x values.
 * @param fy normalization factor for y values.
 * @param n number of points.
 */
extern void psPlError(double *x, double *y, double *e, double fx, double fy, int n);
/**
 * Plots ordinary function y = f(x).
 * @param xfmin start x value in user units.
 * @param xfmax end x value in user units.
 * @param dx step size of x values in user units.
 * @param fx normalization factor for x values.
 * @param fy normalization factor for y values.
 * @param fkt pointer to function.
 * @param p array with function parameters.
 */
extern void psPlFun(double xfmin, double xfmax, double dx, double fx, double fy,
                    double (_api *func)(double, double FAR *), double FAR *p);
/**
 * Plots parametric function x = f1(t), y = f2(t).
 * @param tmin start t value.
 * @param tmax end t value.
 * @param dt step size of t values.
 * @param fx normalization factor for x values.
 * @param fy normalization factor for y values.
 * @param fktx pointer to function for x values.
 * @param px array with function parameters for fktx.
 * @param fkty pointer to function for y values.
 * @param py array with function parameters for fkty.
 */
extern void psPlParFun(double tmin, double tmax, double dt, double fx, double fy,
                       double (_api *funcx)(double, double FAR *), double FAR *px,
                       double (_api *funcy)(double, double FAR *), double FAR *py);
/** Ends PostScript graphic and closes output stream. */
extern void psExit(void);

#ifdef __cplusplus
}
#endif

#endif
